# Copyright 2015 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from hamcrest import assert_that, is_, none, is_not
from netaddr import IPNetwork, IPAddress

from tests.adapters.configured_test_case import ConfiguredTestCase, skip_on_switches


class VrrpTest(ConfiguredTestCase):
    __test__ = False

    @skip_on_switches("juniper", "juniper_qfx_copper", "dell", "dell_telnet", "dell10g", "dell10g_telnet", "juniper_mx", "arista_http")
    def test_add_and_get_group(self):
        try:
            self.client.add_vlan(2999, name="my-test-vlan")
            self.client.add_ip_to_vlan(2999, IPNetwork("10.10.0.3/27"))

            self.client.add_vrrp_group(
                vlan_number=2999,
                group_id=73,
                ips=[IPAddress("10.10.0.1"), IPAddress("10.10.0.2")],
                priority=110,
                track_id=self.test_vrrp_track_id,
                track_decrement=50,
                hello_interval=5,
                dead_interval=15
            )

            vlan = self.get_vlan_from_list(2999)
            vrrp_group = next((g for g in vlan.vrrp_groups if g.id == 73), None)

            assert_that(vrrp_group, is_not(None))

            assert_that([str(ip) for ip in vrrp_group.ips], is_(["10.10.0.1", "10.10.0.2"]))
            assert_that(vrrp_group.priority, is_(110))
            assert_that(vrrp_group.track_id, is_(self.test_vrrp_track_id))
            assert_that(vrrp_group.track_decrement, is_(50))
            assert_that(vrrp_group.hello_interval, is_(5))
            assert_that(vrrp_group.dead_interval, is_(15))

            self.client.remove_vrrp_group(vlan_number=2999, group_id=73)

            vlan = self.get_vlan_from_list(2999)
            vrrp_group = next((g for g in vlan.vrrp_groups if g.id == 73), None)
            assert_that(vrrp_group, is_(none()))

        finally:
            self.client.remove_vlan(2999)



# Copyright 2015 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from unittest import TestCase
from flexmock import flexmock

from netman.core.objects.access_groups import IN
from netman.core.objects.interface_states import OFF, ON
from netman.core.objects.switch_base import SwitchOperations
from tests import ignore_deprecation_warnings


class BackwardCompatibleSwitchOperationsTest(TestCase):
    def setUp(self):
        self.switch = flexmock(SwitchOperations())

    @ignore_deprecation_warnings
    def test_remove_access_vlan_call_unset_interface_access_vlan(self):
        self.switch.should_receive("unset_interface_access_vlan").with_args(1000).once()
        self.switch.remove_access_vlan(1000)

    @ignore_deprecation_warnings
    def test_configure_native_vlan_call_set_interface_native_vlan(self):
        self.switch.should_receive("set_interface_native_vlan").with_args("ethernet 1/g10", 1000).once()

        self.switch.configure_native_vlan("ethernet 1/g10", 1000)

    @ignore_deprecation_warnings
    def test_remove_native_vlan_call_unset_interface_native_vlan(self):
        self.switch.should_receive("unset_interface_native_vlan").with_args(1000).once()

        self.switch.remove_native_vlan(1000)

    @ignore_deprecation_warnings
    def test_remove_vlan_access_group_call_unset_vlan_access_group(self):
        self.switch.should_receive("unset_vlan_access_group").with_args(1000, IN).once()

        self.switch.remove_vlan_access_group(1000, IN)

    @ignore_deprecation_warnings
    def test_remove_vlan_vrf_call_unset_vlan_vrf(self):
        self.switch.should_receive("unset_vlan_vrf").with_args(1000).once()

        self.switch.remove_vlan_vrf(1000)

    @ignore_deprecation_warnings
    def test_remove_interface_description_call_unset_interface_description(self):
        self.switch.should_receive("unset_interface_description").with_args("ethernet 1/g10").once()

        self.switch.remove_interface_description("ethernet 1/g10")

    @ignore_deprecation_warnings
    def test_remove_bond_description_call_unset_bond_description(self):
        self.switch.should_receive("unset_bond_description").with_args(295).once()

        self.switch.remove_bond_description(295)

    @ignore_deprecation_warnings
    def test_configure_bond_native_vlan_call_set_bond_native_vlan(self):
        self.switch.should_receive("set_bond_native_vlan").with_args(295, 1000).once()

        self.switch.configure_bond_native_vlan(295, 1000)

    @ignore_deprecation_warnings
    def test_remove_bond_native_vlan_call_unset_bond_native_vlan(self):
        self.switch.should_receive("unset_bond_native_vlan").with_args(295).once()

        self.switch.remove_bond_native_vlan(295)

    @ignore_deprecation_warnings
    def test_enable_lldp_call_set_interface_lldp_state(self):
        self.switch.should_receive("set_interface_lldp_state").with_args("ethernet 1/g10", True).once()

        self.switch.enable_lldp("ethernet 1/g10", True)

    @ignore_deprecation_warnings
    def test_shutdown_interface_call_set_interface_state(self):
        self.switch.should_receive("set_interface_state").with_args("ethernet 1/g10", OFF).once()

        self.switch.shutdown_interface("ethernet 1/g10")

    @ignore_deprecation_warnings
    def test_openup_interface_call_set_interface_state(self):
        self.switch.should_receive("set_interface_state").with_args("ethernet 1/g10", ON).once()

        self.switch.openup_interface("ethernet 1/g10")



from hamcrest import is_, assert_that
from netman.core.objects.exceptions import BondAlreadyExist, BadBondNumber
from tests.adapters.compliance_test_case import ComplianceTestCase


class AddBondTest(ComplianceTestCase):
    _dev_sample = "juniper"

    def test_creates_the_bond(self):
        self.client.add_bond(42)

        bond_from_get_bond = self.client.get_bond(42)

        assert_that(bond_from_get_bond.number, is_(42))

        self.janitor.remove_bond(42)

    def test_raises_if_bond_already_exists(self):
        self.client.add_bond(42)

        with self.assertRaises(BondAlreadyExist):
            self.client.add_bond(42)

        self.janitor.remove_bond(42)

    def test_raises_if_bond_number_is_out_of_range(self):
        with self.assertRaises(BadBondNumber):
            self.client.add_bond(1000)



# Copyright 2015 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from netaddr import IPNetwork, IPAddress

from netman.api.objects import vrrp_group
from netman.core.objects.access_groups import IN, OUT
from netman.core.objects.vlan import Vlan


def to_api(vlan):
    return dict(
        number=vlan.number,
        name=vlan.name,
        ips=serialize_ip_network(vlan.ips),
        vrrp_groups=sorted([vrrp_group.to_api(group) for group in vlan.vrrp_groups], key=lambda i: i['id']),
        vrf_forwarding=vlan.vrf_forwarding,
        access_groups={
            "in": vlan.access_groups[IN],
            "out": vlan.access_groups[OUT]
        },
        dhcp_relay_servers=[str(server) for server in vlan.dhcp_relay_servers],
        arp_routing=vlan.arp_routing,
        icmp_redirects=vlan.icmp_redirects,
        unicast_rpf_mode=vlan.unicast_rpf_mode,
        ntp=vlan.ntp,
        varp_ips=serialize_ip_network(vlan.varp_ips),
        load_interval=vlan.load_interval,
        mpls_ip=vlan.mpls_ip
    )


def to_core(serialized):
    access_groups = serialized.pop('access_groups')
    ips = serialized.pop('ips')
    vrrp_groups = serialized.pop('vrrp_groups')
    dhcp_relay_servers = serialized.pop('dhcp_relay_servers')
    varp_ips = serialized.pop('varp_ips')
    return Vlan(
        access_group_in=access_groups['in'],
        access_group_out=access_groups['out'],
        ips=deserialize_ip_network(ips),
        vrrp_groups=[vrrp_group.to_core(group) for group in vrrp_groups],
        dhcp_relay_servers=[IPAddress(i) for i in dhcp_relay_servers],
        varp_ips=deserialize_ip_network(varp_ips),
        **serialized
    )


def deserialize_ip_network(ips):
    return [IPNetwork('{address}/{mask}'.format(**ip)) for ip in ips]


def serialize_ip_network(ips):
    return sorted([{'address': ipn.ip.format(), 'mask': ipn.prefixlen} for ipn in ips], key=lambda i: i['address'])



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import SkipTest

from tests import available_models
from tests.adapters.configured_test_case import ConfiguredTestCase
import sys


class ComplianceTestCase(ConfiguredTestCase):
    class __metaclass__(type):
        def __new__(mcs, name, bases, attrs):
            test_class = type.__new__(mcs, name, bases, attrs)
            if bases[0].__name__ == "ComplianceTestCase":
                _create_associated_test_classes(name, test_class)
            return test_class


def _create_associated_test_classes(test_case_name, test_class):
    test_class.__test__ = False
    for _specs in available_models:
        class_name = "{}{}".format(_specs["switch_descriptor"].model.capitalize(), test_case_name)
        new_test_class = type(class_name, (test_class,), {"__test__": True, "switch_specs": _specs})
        setattr(sys.modules[test_class.__module__], class_name,
                _wrap_tests_with_not_implemented_tolerance(new_test_class))


def _wrap_tests_with_not_implemented_tolerance(test_class):
    for method in dir(test_class):
        if method.startswith("test_"):
            _wrap_test_method(method, test_class)
        elif method == "setUp":
            _wrap_setup_method(method, test_class)
    return test_class


def _wrap_test_method(method, test_class):
    old_method = getattr(test_class, method)

    def wrapper(obj):
        try:
            old_method(obj)
        except NotImplementedError:
            raise SkipTest("Method is not implemented")
    wrapper.__name__ = method
    setattr(test_class, method, wrapper)


def _wrap_setup_method(method, test_class):
    old_method = getattr(test_class, method)

    def wrapper(obj):
        try:
            old_method(obj)
        except NotImplementedError:
            obj.tearDown()
            raise SkipTest("Method is not implemented")

    wrapper.__name__ = method
    setattr(test_class, method, wrapper)



# Copyright 2015 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import tempfile
import textwrap
import unittest

import MockSSH
from hamcrest import equal_to, assert_that, is_, starts_with
from mock import patch, Mock

from twisted.internet.protocol import Factory

from netman.adapters import shell
from netman.adapters.shell.ssh import SshClient
from netman.adapters.shell.telnet import TelnetClient
from netman.core.objects.exceptions import CouldNotConnect, CommandTimeout, ConnectTimeout
from tests.adapters.shell.mock_telnet import MockTelnet
from tests.adapters.shell.mock_terminal_commands import passwd_change_protocol_prompt, passwd_write_password_to_transport, \
    HangingCommand, MultiAsyncWriteCommand, SkippingLineCommand, exit_command_success, KeystrokeAnsweredCommand, \
    AmbiguousCommand

command_passwd = MockSSH.PromptingCommand(
    name='passwd',
    password='1234',
    prompt="Password:",
    success_callbacks=[passwd_change_protocol_prompt],
    failure_callbacks=[passwd_write_password_to_transport])

command_hello = MockSSH.ArgumentValidatingCommand(
    name='hello',
    success_callbacks=[lambda instance: instance.writeln("Bonjour")],
    failure_callbacks=[lambda instance: instance.writeln("Nope")])

command_hang = HangingCommand(name='hang', hang_time=1.1)

command_flush = MultiAsyncWriteCommand(name='flush', count=5, interval=0.1)

command_skips = SkippingLineCommand(name='skips', lines=5)

command_exit = MockSSH.ArgumentValidatingCommand('exit',
                                                 [exit_command_success],
                                                 [lambda instance: instance.writeln("Nope")],
                                                 *[])

command_question = KeystrokeAnsweredCommand('keystroke')

abiguous_command = AmbiguousCommand('ambiguous')

commands = [command_passwd, command_hello, command_hang, command_flush,
            command_skips, command_exit, command_question, abiguous_command]
users = {'admin': '1234'}


class TerminalClientTest(unittest.TestCase):
    __test__ = False

    client = None
    port = None

    def test_connect(self):
        client = self.client("127.0.0.1", "admin", "1234", self.port)
        res = client.do('hello')
        assert_that(res, equal_to(['Bonjour']))

    def test_connect_unicode(self):
        client = self.client(u"127.0.0.1", u"admin", u"1234", self.port)
        res = client.do(u'hello')
        assert_that(res, equal_to(['Bonjour']))

    def test_connect_timeout(self):
        with self.assertRaises(ConnectTimeout) as expect:
            self.client("10.0.0.0", "whatever", "whatever", connect_timeout=1)

        assert_that(str(expect.exception), starts_with("Timed out while connecting to 10.0.0.0 on port"))

    def test_connect_error(self):
        with self.assertRaises(CouldNotConnect) as expect:
            self.client("url.invalid", "bleh", "blah", 12345)

        assert_that(str(expect.exception), is_("Could not connect to url.invalid on port 12345"))

    def test_connect_and_salute(self):
        client = self.client("127.0.0.1", "admin", "1234", self.port)
        client.do('passwd', wait_for="Password:")
        client.do('1234')
        res = client.do('hello')
        assert_that(res, equal_to(['Bonjour']))

    def test_timeout(self):
        client = self.client("127.0.0.1", "admin", "1234", self.port, command_timeout=1)
        client.do('passwd', wait_for="Password:")
        client.do('1234')
        with self.assertRaises(CommandTimeout):
            client.do('hang')

    def test_async_result(self):
        client = self.client("127.0.0.1", "admin", "1234", self.port)
        client.do('passwd', wait_for="Password:")
        client.do('1234')
        res = client.do('flush')
        assert_that(res, equal_to([
            "Line 1",
            "Line 2",
            "Line 3",
            "Line 4",
            "Line 5",
            ]))

    def test_empty_lines_are_filtered_out(self):
        client = self.client("127.0.0.1", "admin", "1234", self.port)
        res = client.do('skips')
        assert_that(res, equal_to(["5 lines skipped!"]))

    def test_read_prompt(self):
        client = self.client("127.0.0.1", "admin", "1234", self.port)
        res = client.get_current_prompt()
        assert_that(res, equal_to("hostname>"))

        client.do('passwd', wait_for="Password:")
        client.do('1234')

        res = client.get_current_prompt()
        assert_that(res, equal_to("hostname#"))

    def test_exit(self):
        client = self.client("127.0.0.1", "admin", "1234", self.port)
        client.do('passwd', wait_for="Password:")
        client.do('1234')
        res = client.do('hello')
        assert_that(res, equal_to(['Bonjour']))

        client.quit('exit')

    def test_chunked_reading(self):
        client = self.client("127.0.0.1", "admin", "1234", self.port, reading_chunk_size=1)
        res = client.do('hello')
        assert_that(res, equal_to(['Bonjour']))

    def test_a_complete_log_of_the_conversation_is_kept_and_up_to_date(self):
        client = self.client("127.0.0.1", "admin", "1234", self.port)
        client.do('passwd', wait_for="Password:")
        client.do('1234')
        res = client.do('hello')
        assert_that(res, equal_to(['Bonjour']))

        client.quit('exit')

        assert_that(client.full_log.replace("\r\n", "\n"), equal_to(textwrap.dedent("""\
            hostname>passwd
            Password:
            hostname#hello
            Bonjour
            hostname#""")))

    def test_send_a_keystroke(self):
        client = self.client("127.0.0.1", "admin", "1234", port=self.port)
        res = client.do('keystroke', wait_for="?", include_last_line=True)
        assert_that(res[-1], is_("whatup?"))
        res = client.send_key('k', wait_for=">", include_last_line=True)
        assert_that(res, equal_to(['K pressed', "hostname>"]))

        client.quit('exit')

        assert_that(client.full_log.replace("\r\n", "\n"), equal_to(textwrap.dedent("""\
            hostname>keystroke
            whatup?k
            K pressed
            hostname>""")))

    def test_support_regex(self):
        client = self.client("127.0.0.1", "admin", "1234", port=self.port)
        res = client.do('ambiguous', wait_for=('>', '#'))
        client.quit('exit')

        assert_that(res, equal_to(['working -> done!']))

    def _get_some_credentials(self):
        return {'host': "host.com", 'username': "user", 'password': "pass"}


class SshClientTest(TerminalClientTest):
    __test__ = True

    client = SshClient
    port = 10010

    @patch('netman.adapters.shell.ssh.SshClient._open_channel')
    def test_changing_default_connect_timeout(self, open_channel_method_mock):
        shell.default_connect_timeout = 60

        SshClient(**self._get_some_credentials())
        self.assertEqual(60, open_channel_method_mock.call_args[0][4])

        shell.default_connect_timeout = 120

        SshClient(**self._get_some_credentials())
        self.assertEqual(120, open_channel_method_mock.call_args[0][4])

    @patch('netman.adapters.shell.ssh.SshClient._open_channel', Mock())
    def test_changing_default_command_timeout(self):
        shell.base.default_command_timeout = 300

        ssh = SshClient(**self._get_some_credentials())
        self.assertEqual(300, ssh.command_timeout)

        shell.default_command_timeout = 600

        ssh2 = SshClient(**self._get_some_credentials())
        self.assertEqual(600, ssh2.command_timeout)


class TelnetClientTest(TerminalClientTest):
    __test__ = True

    client = TelnetClient
    port = 10011

    @patch('netman.adapters.shell.telnet.telnetlib.Telnet')
    @patch('netman.adapters.shell.telnet.TelnetClient._login', Mock())
    def test_changing_default_connect_timeout(self, connect_method_mock):
        shell.default_connect_timeout = 60

        TelnetClient(**self._get_some_credentials())
        self.assertEqual(60, connect_method_mock.call_args[0][2])

        shell.default_connect_timeout = 120

        TelnetClient(**self._get_some_credentials())
        self.assertEqual(120, connect_method_mock.call_args[0][2])

    @patch('netman.adapters.shell.telnet.telnetlib.Telnet', Mock())
    @patch('netman.adapters.shell.telnet.TelnetClient._login', Mock())
    def test_changing_default_command_timeout(self):
        shell.default_command_timeout = 300

        telnet = TelnetClient(**self._get_some_credentials())
        self.assertEqual(300, telnet.command_timeout)

        shell.default_command_timeout = 600

        telnet2 = TelnetClient(**self._get_some_credentials())
        self.assertEqual(600, telnet2.command_timeout)


class SwitchTelnetFactory(Factory):
    def __init__(self, prompt, commands):
        self.prompt = prompt
        self.commands = commands

    def protocol(self):
        return MockTelnet(prompt=self.prompt, commands=self.commands)


def telnet_hook_to_reactor(reactor):
    reactor.listenTCP(interface="127.0.0.1", port=TelnetClientTest.port, factory=SwitchTelnetFactory("hostname>", commands))


def ssh_hook_to_reactor(reactor):
    sshFactory = MockSSH.getSSHFactory(commands,
                                       "hostname>",
                                       tempfile.mkdtemp(),
                                       **users)
    reactor.listenTCP(interface='127.0.0.1', port=SshClientTest.port, factory=sshFactory)



# Copyright 2015 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from hamcrest import equal_to, assert_that, has_length, is_, none
from netaddr import IPNetwork, IPAddress

from netman.core.objects.access_groups import IN, OUT
from netman.core.objects.exceptions import UnknownVlan, UnknownInterface, \
    UnknownResource
from netman.core.objects.interface_states import OFF, ON
from netman.core.objects.port_modes import ACCESS, TRUNK
from tests.adapters.configured_test_case import ConfiguredTestCase, skip_on_switches


class VlanManagementTest(ConfiguredTestCase):
    __test__ = False

    @skip_on_switches("juniper", "juniper_qfx_copper", "dell", "dell_telnet", "dell10g", "dell10g_telnet", "juniper_mx", "arista_http")
    def test_get_vlan(self):
        self.client.add_vlan(2999, name="my-test-vlan")
        self.client.set_vlan_access_group(2999, IN, "ACL-IN")
        self.client.set_vlan_access_group(2999, OUT, "ACL-OUT")
        self.client.set_vlan_vrf(2999, "DEFAULT-LAN")
        self.client.add_ip_to_vlan(2999, IPNetwork("10.10.10.2/29"))
        self.client.add_vrrp_group(vlan_number=2999, group_id=73, ips=[IPAddress("10.10.10.1")], priority=110,
                                   track_id=self.test_vrrp_track_id, track_decrement=50, hello_interval=5, dead_interval=15)
        self.client.add_dhcp_relay_server(2999, IPAddress("10.10.10.11"))
        self.try_to.set_vlan_icmp_redirects_state(2999, False)
        self.try_to.set_vlan_ntp_state(2999, False)

        single_vlan = self.client.get_vlan(2999)
        vlan_from_list = self.get_vlan_from_list(2999)

        assert_that(single_vlan, equal_to(vlan_from_list))

    @skip_on_switches("juniper", "juniper_qfx_copper", "dell", "dell_telnet", "dell10g", "dell10g_telnet", "juniper_mx", "arista_http")
    def test_get_vlan_defaults(self):
        self.client.add_vlan(2999, name="my-test-vlan")

        single_vlan = self.client.get_vlan(2999)
        vlan_from_list = self.get_vlan_from_list(2999)

        assert_that(single_vlan, equal_to(vlan_from_list))

    @skip_on_switches("juniper", "juniper_qfx_copper", "dell", "dell_telnet", "dell10g", "dell10g_telnet", "juniper_mx", "arista_http")
    def test_get_vlan_fails(self):
        with self.assertRaises(UnknownVlan):
            self.client.get_vlan(4000)

    @skip_on_switches("juniper_mx", "arista_http")
    def test_adding_and_removing_a_vlan(self):
        self.client.add_vlan(2999, name="my-test-vlan")

        vlan = self.get_vlan_from_list(2999)
        assert_that(vlan.name, equal_to('my-test-vlan'))
        assert_that(len(vlan.ips), equal_to(0))

        self.client.remove_vlan(2999)

        vlans = self.client.get_vlans()
        vlan = next((vlan for vlan in vlans if vlan.number == 2999), None)
        assert_that(vlan is None)

    @skip_on_switches("juniper_mx", "arista_http")
    def test_setting_a_vlan_on_an_interface(self):
        self.client.add_vlan(2999, name="my-test-vlan")

        self.client.set_access_mode(self.test_port)

        self.client.set_access_vlan(self.test_port, vlan=2999)

        self.client.unset_interface_access_vlan(self.test_port)

        self.client.remove_vlan(2999)

    @skip_on_switches("juniper_mx", "arista_http")
    def test_port_mode_trunk(self):
        self.client.add_vlan(2999, name="my-test-vlan")

        self.client.set_trunk_mode(self.test_port)

        self.client.remove_vlan(2999)

    @skip_on_switches("juniper_mx", "arista_http")
    def test_port_mode_access(self):
        self.client.add_vlan(2999, name="my-test-vlan")

        self.client.set_access_mode(self.test_port)

        self.client.remove_vlan(2999)

    @skip_on_switches("juniper_mx", "arista_http")
    def test_native_trunk(self):
        self.client.add_vlan(2999, name="my-test-vlan")

        self.client.set_trunk_mode(self.test_port)

        self.client.set_interface_native_vlan(self.test_port, vlan=2999)

        self.client.unset_interface_native_vlan(self.test_port)

        self.client.set_access_mode(self.test_port)

        self.client.remove_vlan(2999)

    @skip_on_switches("juniper_mx", "arista_http")
    def test_passing_from_trunk_mode_to_access_gets_rid_of_stuff_in_trunk_mode(self):
        self.client.add_vlan(1100)
        self.client.add_vlan(1200)
        self.client.add_vlan(1300)
        self.client.add_vlan(1400)

        self.client.set_trunk_mode(self.test_port)
        self.client.set_interface_native_vlan(self.test_port, vlan=1200)
        self.client.add_trunk_vlan(self.test_port, vlan=1100)
        self.client.add_trunk_vlan(self.test_port, vlan=1300)
        self.client.add_trunk_vlan(self.test_port, vlan=1400)

        interfaces = self.client.get_interfaces()
        test_if = next(i for i in interfaces if i.name == self.test_port)

        assert_that(test_if.port_mode, equal_to(TRUNK))
        assert_that(test_if.trunk_native_vlan, equal_to(1200))
        assert_that(test_if.access_vlan, equal_to(None))
        assert_that(test_if.trunk_vlans, equal_to([1100, 1300, 1400]))

        self.client.set_access_mode(self.test_port)

        interfaces = self.client.get_interfaces()
        test_if = next(i for i in interfaces if i.name == self.test_port)

        assert_that(test_if.port_mode, equal_to(ACCESS))
        assert_that(test_if.trunk_native_vlan, equal_to(None))
        assert_that(test_if.access_vlan, equal_to(None))
        assert_that(test_if.trunk_vlans, has_length(0))

        self.client.remove_vlan(1100)
        self.client.remove_vlan(1200)
        self.client.remove_vlan(1300)
        self.client.remove_vlan(1400)

    @skip_on_switches("juniper_mx", "arista_http")
    def test_invalid_vlan_parameter_fails(self):
        with self.assertRaises(UnknownVlan):
            self.client.remove_vlan(2999)

        with self.assertRaises(UnknownVlan):
            self.client.set_access_vlan(self.test_port, vlan=2999)

        self.client.set_trunk_mode(self.test_port)

        with self.assertRaises(UnknownVlan):
            self.client.add_trunk_vlan(self.test_port, vlan=2999)

        with self.assertRaises(UnknownVlan):
            self.client.set_interface_native_vlan(self.test_port, vlan=2999)

        with self.assertRaises(UnknownVlan):
            self.client.add_trunk_vlan(self.test_port, vlan=2999)

        self.client.add_vlan(2999, name="my-test-vlan")
        # TODO(jprovost) Unify switch adapters to raise the same exception
        with self.assertRaises(UnknownResource):
            self.client.remove_trunk_vlan(self.test_port, vlan=2999)

    @skip_on_switches("juniper", "juniper_qfx_copper", "juniper_mx", "arista_http")
    def test_invalid_interface_parameter_fails(self):
        with self.assertRaises(UnknownInterface):
            self.client.set_interface_state('42/9999', ON)

        with self.assertRaises(UnknownInterface):
            self.client.set_interface_state('42/9999', OFF)

        with self.assertRaises(UnknownInterface):
            self.client.set_access_mode('42/9999')

        with self.assertRaises(UnknownInterface):
            self.client.set_trunk_mode('42/9999')

        # TODO(jprovost) Unify switch adapters to raise the same exception
        with self.assertRaises(UnknownResource):
            self.client.set_access_vlan('42/9999', 1234)

        with self.assertRaises(UnknownInterface):
            self.client.unset_interface_access_vlan('42/9999')

        # TODO(jprovost) Unify switch adapters to raise the same exception
        # Dell 10G raises UnknownInterface
        # other raises UnknownVlan
        with self.assertRaises(UnknownResource):
            self.client.add_trunk_vlan('42/9999', 2999)

        # TODO(jprovost) Behavior is inconsistent across switch adapters
        # Brocade raises UnknownInterface
        # Dell 10G raises NativeVlanNotSet
        # Cisco does not raise
        try:
            self.client.unset_interface_native_vlan(self.test_port)
        except UnknownResource:
            pass

        self.client.add_vlan(2999, name="my-test-vlan")

        with self.assertRaises(UnknownInterface):
            self.client.set_access_vlan('42/9999', 2999)

        # TODO(jprovost) Unify switch adapters to raise the same exception
        # Dell 10G raises UnknownInterface
        # other raises TrunkVlanNotSet
        with self.assertRaises(UnknownResource):
            self.client.remove_trunk_vlan('42/9999', 2999)

        with self.assertRaises(UnknownInterface):
            self.client.set_interface_native_vlan('42/9999', 2999)

    @skip_on_switches("juniper", "juniper_qfx_copper", "dell", "dell_telnet", "dell10g", "dell10g_telnet", "juniper_mx", "arista_http")
    def test_vrf_management(self):

        self.client.add_vlan(2999, name="my-test-vlan")

        self.client.set_vlan_vrf(2999, 'DEFAULT-LAN')

        vlan = self.get_vlan_from_list(2999)
        assert_that(vlan.vrf_forwarding, is_('DEFAULT-LAN'))

        self.client.unset_vlan_vrf(2999)

        vlan = self.get_vlan_from_list(2999)
        assert_that(vlan.vrf_forwarding, is_(none()))

    def tearDown(self):
        self.janitor.remove_vlan(2999)
        self.janitor.set_access_mode(self.test_port)
        super(VlanManagementTest, self).tearDown()



# Copyright 2015 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from netman.api.objects import sub_dict, Serializer, Serializers
from netman.core.objects.interface import BaseInterface
from netman.core.objects.port_modes import ACCESS, TRUNK, DYNAMIC, BOND_MEMBER

__all__ = ['to_api', 'to_core']


serialized_port_mode = {
    ACCESS: "access",
    TRUNK: "trunk",
    DYNAMIC: "dynamic",
    BOND_MEMBER: "bond_member"
}


class V1(Serializer):
    since_version = 1

    def to_core(self, serialized):
        return BaseInterface(
            port_mode=dict((v, k) for k, v in serialized_port_mode.iteritems())[serialized.pop('port_mode')],
            **sub_dict(serialized, 'shutdown', 'access_vlan', 'trunk_native_vlan', 'trunk_vlans', 'mtu')
        )

    def to_api(self, base_interface):
        return dict(
            shutdown=base_interface.shutdown,
            port_mode=serialized_port_mode[base_interface.port_mode],
            access_vlan=base_interface.access_vlan,
            trunk_native_vlan=base_interface.trunk_native_vlan,
            trunk_vlans=sorted(base_interface.trunk_vlans),
            mtu=base_interface.mtu
        )


serializers = Serializers(V1())

to_api = serializers.to_api
to_core = serializers.to_core



# Copyright 2015 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import textwrap
import unittest

import mock
from flexmock import flexmock, flexmock_teardown
from hamcrest import assert_that, equal_to, is_, instance_of

from netman.adapters.switches import juniper
from netman.adapters.switches.juniper.base import Juniper
from netman.core.objects.exceptions import UnknownInterface
from netman.core.objects.port_modes import ACCESS, TRUNK, BOND_MEMBER
from netman.core.objects.switch_descriptor import SwitchDescriptor
from netman.core.objects.switch_transactional import FlowControlSwitch
from tests import ignore_deprecation_warnings
from tests.adapters.switches.juniper_test import an_ok_response, is_xml, a_configuration, an_rpc_response


@ignore_deprecation_warnings
def test_factory():
    lock = mock.Mock()
    switch = juniper.qfx_copper_factory(SwitchDescriptor(hostname='hostname', model='juniper_qfx_copper', username='username', password='password', port=22), lock)

    assert_that(switch, instance_of(FlowControlSwitch))
    assert_that(switch.wrapped_switch, instance_of(Juniper))
    assert_that(switch.lock, is_(lock))
    assert_that(switch.switch_descriptor.hostname, equal_to("hostname"))
    assert_that(switch.switch_descriptor.model, equal_to("juniper_qfx_copper"))
    assert_that(switch.switch_descriptor.username, equal_to("username"))
    assert_that(switch.switch_descriptor.password, equal_to("password"))
    assert_that(switch.switch_descriptor.port, equal_to(22))


class JuniperTest(unittest.TestCase):

    def setUp(self):
        self.switch = juniper.qfx_copper.netconf(SwitchDescriptor(model='juniper', hostname="toto"))

        self.netconf_mock = flexmock()
        self.switch.netconf = self.netconf_mock
        self.switch.in_transaction = True

    def tearDown(self):
        flexmock_teardown()

    def test_get_interface(self):
        self.switch.in_transaction = False

        self.netconf_mock.should_receive("get_config").with_args(source="running", filter=is_xml("""
                <filter>
                  <configuration>
                     <interfaces>
                       <interface>
                          <name>ge-0/0/1</name>
                        </interface>
                      </interfaces>
                    <vlans />
                  </configuration>
                </filter>
            """)).and_return(a_configuration("""
                <interfaces>
                  <interface>
                    <name>ge-0/0/1</name>
                    <unit>
                      <name>0</name>
                      <family>
                        <ethernet-switching>
                        </ethernet-switching>
                      </family>
                    </unit>
                  </interface>
                  <interface>
                    <name>ge-0/0/2</name>
                    <disable />
                    <description>Howdy</description>
                    <unit>
                      <name>0</name>
                      <family>
                        <ethernet-switching>
                          <vlan>
                            <members>1000</members>
                          </vlan>
                        </ethernet-switching>
                      </family>
                    </unit>
                  </interface>
                </interfaces>
                <vlans/>
            """))

        if1 = self.switch.get_interface('ge-0/0/1')

        assert_that(if1.name, equal_to("ge-0/0/1"))
        assert_that(if1.shutdown, equal_to(False))
        assert_that(if1.port_mode, equal_to(ACCESS))
        assert_that(if1.access_vlan, equal_to(None))
        assert_that(if1.trunk_native_vlan, equal_to(None))
        assert_that(if1.trunk_vlans, equal_to([]))
        assert_that(if1.auto_negotiation, equal_to(None))

    def test_get_nonexistent_interface_raises(self):
        self.switch.in_transaction = False
        self.netconf_mock.should_receive("get_config").with_args(source="running", filter=is_xml("""
                    <filter>
                      <configuration>
                          <interfaces>
                            <interface>
                              <name>ge-0/0/INEXISTENT</name>
                            </interface>
                          </interfaces>
                        <vlans />
                      </configuration>
                    </filter>
                """)).and_return(a_configuration("""
                    <interfaces/>
                    <vlans/>
                """))
        self.netconf_mock.should_receive("rpc").with_args(is_xml("""
                    <get-interface-information>
                      <terse/>
                    </get-interface-information>
                """)).and_return(an_rpc_response(textwrap.dedent("""
                    <interface-information style="terse">
                      <physical-interface>
                        <name>
                    ge-0/0/1
                    </name>
                        <admin-status>
                    down
                    </admin-status>
                        <oper-status>
                    down
                    </oper-status>
                      </physical-interface>
                    </interface-information>
                """)))

        with self.assertRaises(UnknownInterface) as expect:
            self.switch.get_interface('ge-0/0/INEXISTENT')

        assert_that(str(expect.exception), equal_to("Unknown interface ge-0/0/INEXISTENT"))

    def test_get_interfaces(self):
        self.switch.in_transaction = False

        self.netconf_mock.should_receive("rpc").with_args(is_xml("""
                    <get-interface-information>
                      <terse/>
                    </get-interface-information>
                """)).and_return(an_rpc_response(textwrap.dedent("""
                    <interface-information style="terse">
                      <physical-interface>
                        <name>
                    ge-0/0/1
                    </name>
                        <admin-status>
                    up
                    </admin-status>
                        <oper-status>
                    down
                    </oper-status>
                        <logical-interface>
                          <name>
                    ge-0/0/1.0
                    </name>
                          <admin-status>
                    up
                    </admin-status>
                          <oper-status>
                    down
                    </oper-status>
                          <filter-information>
                          </filter-information>
                          <address-family>
                            <address-family-name>
                    eth-switch
                    </address-family-name>
                          </address-family>
                        </logical-interface>
                      </physical-interface>
                      <physical-interface>
                        <name>
                    ge-0/0/2
                    </name>
                        <admin-status>
                    down
                    </admin-status>
                        <oper-status>
                    down
                    </oper-status>
                        <logical-interface>
                          <name>
                    ge-0/0/2.0
                    </name>
                          <admin-status>
                    up
                    </admin-status>
                          <oper-status>
                    down
                    </oper-status>
                          <filter-information>
                          </filter-information>
                          <address-family>
                            <address-family-name>
                    eth-switch
                    </address-family-name>
                          </address-family>
                        </logical-interface>
                      </physical-interface>
                      <physical-interface>
                        <name>
                    ge-0/0/3
                    </name>
                        <admin-status>
                    up
                    </admin-status>
                        <oper-status>
                    down
                    </oper-status>
                        <logical-interface>
                          <name>
                    ge-0/0/3.0
                    </name>
                          <admin-status>
                    up
                    </admin-status>
                          <oper-status>
                    down
                    </oper-status>
                          <filter-information>
                          </filter-information>
                          <address-family>
                            <address-family-name>
                    eth-switch
                    </address-family-name>
                          </address-family>
                        </logical-interface>
                      </physical-interface>
                      <physical-interface>
                        <name>
                    ge-0/0/4
                    </name>
                        <admin-status>
                    up
                    </admin-status>
                        <oper-status>
                    down
                    </oper-status>
                        <logical-interface>
                          <name>
                    ge-0/0/4.0
                    </name>
                          <admin-status>
                    up
                    </admin-status>
                          <oper-status>
                    down
                    </oper-status>
                          <filter-information>
                          </filter-information>
                          <address-family>
                            <address-family-name>
                    eth-switch
                    </address-family-name>
                          </address-family>
                        </logical-interface>
                      </physical-interface>
                      <physical-interface>
                        <name>
                    ge-0/0/5
                    </name>
                        <admin-status>
                    up
                    </admin-status>
                        <oper-status>
                    down
                    </oper-status>
                      </physical-interface>
                    </interface-information>
                """)))

        self.netconf_mock.should_receive("get_config").with_args(source="running", filter=is_xml("""
            <filter>
              <configuration>
                <interfaces />
                <vlans />
              </configuration>
            </filter>
        """)).and_return(a_configuration("""
            <interfaces>
              <interface>
                <name>ge-0/0/1</name>
                <unit>
                  <name>0</name>
                  <family>
                    <ethernet-switching>
                    </ethernet-switching>
                  </family>
                </unit>
              </interface>
              <interface>
                <name>ge-0/0/2</name>
                <disable />
                <description>Howdy</description>
                <unit>
                  <name>0</name>
                  <family>
                    <ethernet-switching>
                      <vlan>
                        <members>1000</members>
                      </vlan>
                    </ethernet-switching>
                  </family>
                </unit>
              </interface>
              <interface>
                <name>ge-0/0/3</name>
                <native-vlan-id>2000</native-vlan-id>
                <unit>
                  <name>0</name>
                  <family>
                    <ethernet-switching>
                      <interface-mode>trunk</interface-mode>
                      <vlan>
                        <members>999-1001</members>
                        <members>1000</members>
                      </vlan>
                    </ethernet-switching>
                  </family>
                </unit>
              </interface>
              <interface>
                <name>ge-0/0/4</name>
                <ether-options>
                  <no-auto-negotiation/>
                </ether-options>
                <unit>
                  <name>0</name>
                  <family>
                    <ethernet-switching>
                      <interface-mode>trunk</interface-mode>
                    </ethernet-switching>
                  </family>
                </unit>
              </interface>
              <interface>
                <name>ge-0/0/5</name>
                <ether-options>
                  <auto-negotiation/>
                  <ieee-802.3ad>
                    <bundle>ae10</bundle>
                  </ieee-802.3ad>
                </ether-options>
              </interface>
              <interface>
                <name>vlan</name>
                <unit>
                  <name>40</name>
                </unit>
              </interface>
              <interface>
                <name>ae10</name>
                <aggregated-ether-options>
                  <lacp>
                    <active/>
                    <periodic>slow</periodic>
                  </lacp>
                </aggregated-ether-options>
                <unit>
                  <name>0</name>
                  <family>
                    <ethernet-switching />
                  </family>
                </unit>
              </interface>
            </interfaces>
            <vlans/>
        """))

        if1, if2, if3, if4, if5 = self.switch.get_interfaces()

        assert_that(if1.name, equal_to("ge-0/0/1"))
        assert_that(if1.shutdown, equal_to(False))
        assert_that(if1.port_mode, equal_to(ACCESS))
        assert_that(if1.access_vlan, equal_to(None))
        assert_that(if1.trunk_native_vlan, equal_to(None))
        assert_that(if1.trunk_vlans, equal_to([]))
        assert_that(if1.auto_negotiation, equal_to(None))

        assert_that(if2.name, equal_to("ge-0/0/2"))
        assert_that(if2.shutdown, equal_to(True))
        assert_that(if2.port_mode, equal_to(ACCESS))
        assert_that(if2.access_vlan, equal_to(1000))
        assert_that(if2.trunk_native_vlan, equal_to(None))
        assert_that(if2.trunk_vlans, equal_to([]))

        assert_that(if3.name, equal_to("ge-0/0/3"))
        assert_that(if3.port_mode, equal_to(TRUNK))
        assert_that(if3.access_vlan, equal_to(None))
        assert_that(if3.trunk_native_vlan, equal_to(2000))
        assert_that(if3.trunk_vlans, equal_to([999, 1000, 1001]))

        assert_that(if4.name, equal_to("ge-0/0/4"))
        assert_that(if4.trunk_native_vlan, equal_to(None))
        assert_that(if4.trunk_vlans, equal_to([]))
        assert_that(if4.auto_negotiation, equal_to(False))

        assert_that(if5.name, equal_to("ge-0/0/5"))
        assert_that(if5.port_mode, equal_to(BOND_MEMBER))
        assert_that(if5.bond_master, equal_to(10))
        assert_that(if5.auto_negotiation, equal_to(True))

    def test_get_interface_with_trunk_native_vlan_at_root(self):
        self.switch.in_transaction = False

        self.netconf_mock.should_receive("get_config").with_args(source="running", filter=is_xml("""
                <filter>
                  <configuration>
                     <interfaces>
                       <interface>
                          <name>ge-0/0/1</name>
                        </interface>
                      </interfaces>
                    <vlans />
                  </configuration>
                </filter>
            """)).and_return(a_configuration("""
                <interfaces>
                  <interface>
                    <name>ge-0/0/1</name>
                    <native-vlan-id>1000</native-vlan-id>
                    <unit>
                      <name>0</name>
                      <family>
                        <ethernet-switching>
                        </ethernet-switching>
                      </family>
                    </unit>
                  </interface>
                </interfaces>
                <vlans/>
            """))

        if1 = self.switch.get_interface('ge-0/0/1')

        assert_that(if1.name, equal_to("ge-0/0/1"))
        assert_that(if1.trunk_native_vlan, equal_to(1000))

    def test_port_mode_access_with_no_port_mode_or_vlan_set_just_sets_the_port_mode(self):
        self.netconf_mock.should_receive("get_config").with_args(source="candidate", filter=is_xml("""
            <filter>
              <configuration>
                <interfaces/>
                <vlans/>
              </configuration>
            </filter>
        """)).and_return(a_configuration("""
            <interfaces>
              <interface>
                <name>ge-0/0/6</name>
                <unit>
                  <name>0</name>
                  <family>
                    <ethernet-switching>
                    </ethernet-switching>
                  </family>
                </unit>
              </interface>
            </interfaces>
            <vlans/>
        """))

        self.netconf_mock.should_receive("edit_config").once().with_args(target="candidate", config=is_xml("""
            <config>
              <configuration>
                <interfaces>
                  <interface>
                    <name>ge-0/0/6</name>
                    <unit>
                      <name>0</name>
                      <family>
                        <ethernet-switching>
                          <interface-mode>access</interface-mode>
                        </ethernet-switching>
                      </family>
                    </unit>
                  </interface>
                </interfaces>
              </configuration>
            </config>
        """)).and_return(an_ok_response())

        self.switch.set_access_mode("ge-0/0/6")

    def test_add_interface_to_bond(self):
        self.netconf_mock.should_receive("get_config").with_args(source="candidate", filter=is_xml("""
            <filter>
              <configuration>
                <interfaces/>
                <vlans/>
                <protocols>
                  <rstp>
                    <interface />
                  </rstp>
                </protocols>
              </configuration>
            </filter>
        """)).and_return(a_configuration("""
            <interfaces>
              <interface>
                <name>ae10</name>
              </interface>
              <interface>
                <name>ge-0/0/1</name>
              </interface>
            </interfaces>
            <vlans/>
        """))

        self.netconf_mock.should_receive("edit_config").once().with_args(target="candidate", config=is_xml("""
            <config>
              <configuration>
                <interfaces>
                  <interface operation="replace">
                    <name>ge-0/0/1</name>
                    <ether-options>
                      <auto-negotiation/>
                      <ieee-802.3ad>
                        <bundle>ae10</bundle>
                      </ieee-802.3ad>
                    </ether-options>
                  </interface>
                </interfaces>
              </configuration>
            </config>""")).and_return(an_ok_response())

        self.switch.add_interface_to_bond('ge-0/0/1', 10)

    def test_add_interface_to_bond_gets_up_to_speed_and_removes_existing_rstp_protocol(self):
        self.netconf_mock.should_receive("get_config").with_args(source="candidate", filter=is_xml("""
            <filter>
              <configuration>
                <interfaces/>
                <vlans/>
                <protocols>
                  <rstp>
                    <interface />
                  </rstp>
                </protocols>
              </configuration>
            </filter>
        """)).and_return(a_configuration("""
            <interfaces>
              <interface>
                <name>ae10</name>
                <aggregated-ether-options>
                  <link-speed>1g</link-speed>
                </aggregated-ether-options>
              </interface>
              <interface>
                <name>ge-0/0/1</name>
              </interface>
            </interfaces>
            <vlans/>
            <protocols>
              <rstp>
                <interface>
                  <name>ge-0/0/1</name>
                  <edge />
                </interface>
              </rstp>
            </protocols>
        """))

        self.netconf_mock.should_receive("edit_config").once().with_args(target="candidate", config=is_xml("""
            <config>
              <configuration>
                <interfaces>
                  <interface operation="replace">
                    <name>ge-0/0/1</name>
                    <ether-options>
                      <auto-negotiation/>
                      <ieee-802.3ad>
                        <bundle>ae10</bundle>
                      </ieee-802.3ad>
                    </ether-options>
                  </interface>
                </interfaces>
                <protocols>
                  <rstp>
                    <interface operation="delete">
                      <name>ge-0/0/1</name>
                    </interface>
                  </rstp>
                </protocols>
              </configuration>
            </config>""")).and_return(an_ok_response())

        self.switch.add_interface_to_bond('ge-0/0/1', 10)

    def test_change_bond_speed_update_slaves_and_interface_at_same_time(self):
        self.netconf_mock.should_receive("get_config").with_args(source="candidate", filter=is_xml("""
            <filter>
              <configuration>
                <interfaces />
              </configuration>
            </filter>
        """)).and_return(a_configuration("""
            <interfaces>
              <interface>
                <name>ae10</name>
              </interface>
              <interface>
                <name>ge-0/0/1</name>
                <ether-options>
                  <ieee-802.3ad>
                    <bundle>ae10</bundle>
                  </ieee-802.3ad>
                </ether-options>
              </interface>
              <interface>
                <name>ge-0/0/2</name>
              </interface>
            </interfaces>
        """))

        self.netconf_mock.should_receive("edit_config").once().with_args(target="candidate", config=is_xml("""
            <config>
              <configuration>
                <interfaces>
                  <interface>
                    <name>ae10</name>
                    <aggregated-ether-options>
                      <link-speed>1g</link-speed>
                    </aggregated-ether-options>
                  </interface>
                </interfaces>
              </configuration>
            </config>""")).and_return(an_ok_response())

        self.switch.set_bond_link_speed(10, '1g')



# Copyright 2015 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from hamcrest import assert_that, is_
from flexmock import flexmock, flexmock_teardown
from netaddr import IPAddress, IPNetwork

from netman.adapters.switches.cached import CachedSwitch
from netman.core.objects.access_groups import IN, OUT
from netman.core.objects.bond import Bond
from netman.core.objects.interface import Interface
from netman.core.objects.interface_states import OFF, ON
from netman.core.objects.port_modes import ACCESS, TRUNK, BOND_MEMBER
from netman.core.objects.switch_descriptor import SwitchDescriptor
from netman.core.objects.unicast_rpf_modes import STRICT
from netman.core.objects.vlan import Vlan
from netman.core.objects.vrrp_group import VrrpGroup

from tests import ExactIpNetwork


class CacheSwitchTest(unittest.TestCase):
    def setUp(self):
        self.real_switch_mock = flexmock()
        self.real_switch_mock.switch_descriptor = SwitchDescriptor(
            'model', 'hostname')
        self.switch = CachedSwitch(self.real_switch_mock)

    def tearDown(self):
        flexmock_teardown()

    def test_connect(self):
        self.real_switch_mock.should_receive("connect").once()
        self.switch.connect()

    def test_disconnect(self):
        self.real_switch_mock.should_receive("disconnect").once()
        self.switch.disconnect()

    def test_start_transaction(self):
        self.real_switch_mock.should_receive("start_transaction").once()
        self.switch.start_transaction()

    def test_commit_transaction(self):
        self.real_switch_mock.should_receive("commit_transaction").once()
        self.switch.commit_transaction()

    def test_rollback_transaction(self):
        self.real_switch_mock.should_receive("rollback_transaction").once()
        self.switch.rollback_transaction()

    def test_end_transaction(self):
        self.real_switch_mock.should_receive("end_transaction").once()
        self.switch.end_transaction()

    def test_get_vlan(self):
        a_vlan = Vlan(1, 'first')

        self.real_switch_mock.should_receive("get_vlan").with_args(1).once().and_return(
            a_vlan)
        assert_that(self.switch.get_vlan(1), is_(a_vlan))
        assert_that(self.switch.get_vlan(1), is_(a_vlan))

    def test_get_vlan_after_list(self):
        all_vlans = [Vlan(1, 'first'), Vlan(2, 'second')]

        self.real_switch_mock.should_receive("get_vlans").once().and_return(
            all_vlans)
        assert_that(self.switch.get_vlans(), is_(all_vlans))

        assert_that(self.switch.get_vlan(1), is_(all_vlans[0]))
        assert_that(self.switch.get_vlan(2), is_(all_vlans[1]))

    def test_get_vlan_interfaces(self):
        vlan_interfaces = ["port-channel 1", "port-channel 3", "ethernet 0/2"]

        self.real_switch_mock.should_receive("get_vlan_interfaces").with_args(1).once().and_return(
                vlan_interfaces)
        assert_that(self.switch.get_vlan_interfaces(1), is_(vlan_interfaces))
        assert_that(self.switch.get_vlan_interfaces(1), is_(vlan_interfaces))

    def test_access_new_vlan_after_vlan_list(self):
        all_vlans = [Vlan(1, 'first'), Vlan(2, 'second')]
        vlan3 = Vlan(3, 'third', ips=[IPNetwork("2.2.2.2/24")])

        self.real_switch_mock.should_receive("get_vlans").once().and_return(all_vlans)

        self.real_switch_mock.should_receive("add_ip_to_vlan").once().with_args(3, ExactIpNetwork("2.2.2.2/24"))
        self.real_switch_mock.should_receive("get_vlan").with_args(3).once().and_return(vlan3)

        assert_that(self.switch.get_vlans(), is_(all_vlans))

        self.switch.add_ip_to_vlan(3, IPNetwork("2.2.2.2/24"))

        assert_that(self.switch.get_vlan(3), is_(vlan3))

    def test_get_vlans(self):
        all_vlans = [Vlan(1, 'first'), Vlan(2, 'second')]

        self.real_switch_mock.should_receive("get_vlans").once().and_return(
            all_vlans)
        assert_that(self.switch.get_vlans(), is_(all_vlans))
        assert_that(self.switch.get_vlans(), is_(all_vlans))

    def test_add_vlan_first(self):
        all_vlans = [Vlan(1), Vlan(2), Vlan(123, name='allo')]

        self.real_switch_mock.should_receive("add_vlan").once().with_args(123, name='allo')
        self.switch.add_vlan(123, 'allo')

        self.real_switch_mock.should_receive("get_vlans").once().and_return(
            all_vlans)
        assert_that(self.switch.get_vlans(), is_(all_vlans))
        assert_that(self.switch.get_vlans(), is_(all_vlans))

    def test_add_vlan(self):
        self.real_switch_mock.should_receive("add_vlan").once().with_args(123)
        self.switch.add_vlan(123)
        self.real_switch_mock.should_receive("get_vlan").once().with_args(123).and_return(Vlan(number=123, name=""))

        assert_that(self.switch.get_vlan(123).name, is_(""))

    def test_after_get_vlans_was_cached_adding_new_vlans_will_trigger_single_get_vlans_to_fill_the_gaps(self):
        all_vlans = [Vlan(1)]

        self.real_switch_mock.should_receive("get_vlans").once().ordered().and_return(all_vlans)
        assert_that(self.switch.get_vlans(), is_(all_vlans))

        self.real_switch_mock.should_receive("add_vlan").once().with_args(10)
        self.switch.add_vlan(10)
        self.real_switch_mock.should_receive("add_vlan").once().with_args(11)
        self.switch.add_vlan(11)
        self.real_switch_mock.should_receive("add_vlan").once().with_args(12)
        self.switch.add_vlan(12)

        self.real_switch_mock.should_receive("get_vlan").with_args(10).once().and_return(Vlan(10))
        self.real_switch_mock.should_receive("get_vlan").with_args(11).once().and_return(Vlan(11))
        self.real_switch_mock.should_receive("get_vlan").with_args(12).once().and_return(Vlan(12))
        assert_that(self.switch.get_vlans(), is_([Vlan(1), Vlan(10), Vlan(11), Vlan(12)]))
        assert_that(self.switch.get_vlans(), is_([Vlan(1), Vlan(10), Vlan(11), Vlan(12)]))

    def test_add_vlan_after_get_vlans(self):
        all_vlans = [Vlan(1), Vlan(2)]

        self.real_switch_mock.should_receive("get_vlans").once().ordered().and_return(
            all_vlans)
        assert_that(self.switch.get_vlans(), is_(all_vlans))
        assert_that(self.switch.get_vlans(), is_(all_vlans))

        self.real_switch_mock.should_receive("add_vlan").once().with_args(123, name='allo')
        self.switch.add_vlan(123, 'allo')

        self.real_switch_mock.should_receive("get_vlan").once().ordered().and_return(Vlan(123, name='allo'))
        assert_that(self.switch.get_vlans(), is_(all_vlans+[Vlan(123, name='allo')]))
        assert_that(self.switch.get_vlans(), is_(all_vlans+[Vlan(123, name='allo')]))

    def test_remove_vlan(self):
        self.real_switch_mock.should_receive("get_vlans").once().and_return(
            [Vlan(1), Vlan(2)])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive("remove_vlan").once().with_args(1)
        self.switch.remove_vlan(1)

        assert_that(self.switch.get_vlans(), is_([Vlan(2)]))

    def test_remove_vlan_alone(self):
        self.real_switch_mock.should_receive("remove_vlan").once().with_args(1)
        self.switch.remove_vlan(1)

    def test_get_interfaces(self):
        all_interfaces = [Interface('xe-1/0/1'), Interface('xe-1/0/2')]

        self.real_switch_mock.should_receive("get_interfaces").once() \
            .and_return(all_interfaces)
        assert_that(self.switch.get_interfaces(), is_(all_interfaces))
        assert_that(self.switch.get_interfaces(), is_(all_interfaces))

    def test_get_interface(self):
        interface = Interface('xe-1/0/1')

        self.real_switch_mock.should_receive("get_interface").with_args("xe-1/0/1").once() \
            .and_return(interface)
        assert_that(self.switch.get_interface('xe-1/0/1'), is_(interface))
        assert_that(self.switch.get_interface('xe-1/0/1'), is_(interface))

    def test_set_access_vlan(self):
        self.real_switch_mock.should_receive("get_interfaces").once().and_return(
            [Interface('eth0')])
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("set_access_vlan").once() \
            .with_args('eth0', 123)

        self.switch.set_access_vlan('eth0', 123)

        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('eth0', access_vlan=123)])
        )

    def test_reset_interface_invalidate_cache(self):
        self.real_switch_mock.should_receive("get_interface").with_args("eth0").once().\
            and_return([Interface('eth0')])
        self.switch.get_interface('eth0')

        self.real_switch_mock.should_receive("reset_interface").with_args('eth0')
        self.switch.reset_interface('eth0')

        self.real_switch_mock.should_receive("get_interface").with_args("eth0").once(). \
            and_return([Interface('eth0')])
        self.switch.get_interface('eth0')

    def test_unset_interface_access_vlan(self):
        self.real_switch_mock.should_receive("get_interfaces").once().and_return(
            [Interface('eth0', access_vlan=123)])
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("unset_interface_access_vlan").once() \
            .with_args('eth0')

        self.switch.unset_interface_access_vlan('eth0')

        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('eth0')])
        )

    def test_set_trunk_mode(self):
        self.real_switch_mock.should_receive("get_interfaces").once().and_return(
            [Interface('eth0')])
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("set_trunk_mode").once() \
            .with_args('eth0')

        self.switch.set_trunk_mode('eth0')

        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('eth0', port_mode=TRUNK)])
        )

    def test_set_access_mode(self):
        self.real_switch_mock.should_receive("get_interfaces").once().and_return(
            [Interface('eth0', trunk_native_vlan=1200, trunk_vlans=[1, 2, 3])])
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("set_access_mode").once() \
            .with_args('eth0')

        self.switch.set_access_mode('eth0')

        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('eth0', port_mode=ACCESS,
                           trunk_native_vlan=None, trunk_vlans=[])])
        )

    def test_add_trunk_vlan(self):
        self.real_switch_mock.should_receive("get_interfaces").once() \
            .and_return([Interface('xe-1/0/2')])
        assert_that(self.switch.get_interfaces(), is_([Interface('xe-1/0/2')]))

        self.real_switch_mock.should_receive("add_trunk_vlan").once() \
            .with_args('xe-1/0/2', 1)

        self.switch.add_trunk_vlan('xe-1/0/2', 1)

        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('xe-1/0/2', trunk_vlans=[1])]))

    def test_remove_trunk_vlan(self):
        self.real_switch_mock.should_receive("get_interfaces").once() \
            .and_return([Interface('xe-1/0/2', trunk_vlans=[1])])
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("remove_trunk_vlan").once() \
            .with_args('xe-1/0/2', 1)

        self.switch.remove_trunk_vlan('xe-1/0/2', 1)

        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('xe-1/0/2', trunk_vlans=[])]))

    def test_remove_trunk_vlan_on_interface_not_in_cache(self):
        self.real_switch_mock.should_receive("remove_trunk_vlan").once() \
            .with_args('xe-1/0/2', 1)

        self.switch.remove_trunk_vlan('xe-1/0/2', 1)

    def test_set_interface_state_off(self):
        self.real_switch_mock.should_receive("get_interfaces").once() \
            .and_return([Interface('xe-1/0/2', shutdown=False)])
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("set_interface_state").once() \
            .with_args('xe-1/0/2', OFF)

        self.switch.set_interface_state('xe-1/0/2', OFF)

        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('xe-1/0/2', shutdown=True)]))

    def test_set_interface_state_on(self):
        self.real_switch_mock.should_receive("get_interfaces").once() \
            .and_return([Interface('xe-1/0/2')])
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("set_interface_state").once() \
            .with_args('xe-1/0/2', ON)

        self.switch.set_interface_state('xe-1/0/2', ON)

        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('xe-1/0/2', shutdown=False)]))

    def test_unset_interface_state(self):
        self.real_switch_mock.should_receive("get_interfaces").once().and_return([Interface('xe-1/0/2')])
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("unset_interface_state").once().with_args('xe-1/0/2')

        self.real_switch_mock.should_receive("get_interface").once() \
            .with_args('xe-1/0/2').and_return(Interface('xe-1/0/2', shutdown=False))

        self.switch.unset_interface_state('xe-1/0/2')

        assert_that(self.switch.get_interface('xe-1/0/2').shutdown, is_(False))

    def test_set_interface_auto_negotiation_state_off(self):
        self.real_switch_mock.should_receive("get_interfaces").once() \
            .and_return([Interface('xe-1/0/2', auto_negotiation=True)])
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("set_interface_auto_negotiation_state").once() \
            .with_args('xe-1/0/2', OFF)

        self.switch.set_interface_auto_negotiation_state('xe-1/0/2', OFF)

        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('xe-1/0/2', auto_negotiation=False)]))

    def test_set_interface_auto_negotiation_state_on(self):
        self.real_switch_mock.should_receive("get_interfaces").once() \
            .and_return([Interface('xe-1/0/2')])
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("set_interface_auto_negotiation_state").once() \
            .with_args('xe-1/0/2', ON)

        self.switch.set_interface_auto_negotiation_state('xe-1/0/2', ON)

        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('xe-1/0/2', auto_negotiation=True)]))

    def test_unset_interface_auto_negotiation_state(self):
        self.real_switch_mock.should_receive("get_interfaces").once()\
            .and_return([Interface('xe-1/0/2', auto_negotiation=False)])
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("unset_interface_auto_negotiation_state").once().with_args('xe-1/0/2')

        self.switch.unset_interface_auto_negotiation_state('xe-1/0/2')

        assert_that(self.switch.get_interface('xe-1/0/2').auto_negotiation, is_(None))

    def test_set_interface_native_vlan(self):
        self.real_switch_mock.should_receive("get_interfaces").once() \
            .and_return([Interface('xe-1/0/2')])
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("set_interface_native_vlan").once() \
            .with_args('xe-1/0/2', 20)

        self.switch.set_interface_native_vlan('xe-1/0/2', 20)

        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('xe-1/0/2', trunk_native_vlan=20)]))

    def test_unset_interface_native_vlan(self):
        self.real_switch_mock.should_receive("get_interfaces").once() \
            .and_return([Interface('xe-1/0/2', trunk_native_vlan=20)])
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("unset_interface_native_vlan").once() \
            .with_args('xe-1/0/2')

        self.switch.unset_interface_native_vlan('xe-1/0/2')

        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('xe-1/0/2', trunk_native_vlan=None)]))

    def test_add_ip_to_vlan(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(2)])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive("add_ip_to_vlan").once() \
            .with_args(2, ExactIpNetwork("2.2.2.2/24"))

        self.switch.add_ip_to_vlan(2, IPNetwork("2.2.2.2/24"))

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(2, ips=[ExactIpNetwork("2.2.2.2/24")])]))

    def test_remove_ip_from_vlan(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(2, ips=[IPNetwork("2.2.2.2/24"),
                                      IPNetwork("1.1.1.1/24"),
                                      IPNetwork("1.1.1.2/24"),
                                      IPNetwork("1.1.1.3/24"),
                                      IPNetwork("1.1.1.4/24")])])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive("remove_ip_from_vlan").once() \
            .with_args(2, ExactIpNetwork("1.1.1.2/24"))

        self.switch.remove_ip_from_vlan(2, IPNetwork("1.1.1.2/24"))

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(2, ips=[ExactIpNetwork("2.2.2.2/24"),
                              ExactIpNetwork("1.1.1.1/24"),
                              ExactIpNetwork("1.1.1.3/24"),
                              ExactIpNetwork("1.1.1.4/24")])]))

    def test_set_vlan_access_group(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(123)])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive("set_vlan_access_group").once() \
            .with_args(123, IN, 'vlan-access-group')

        self.switch.set_vlan_access_group(123, IN, 'vlan-access-group')

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(123, access_group_in='vlan-access-group')]))

    def test_unset_vlan_access_group(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(123, access_group_out='vlan-access-group')])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive("unset_vlan_access_group").once() \
            .with_args(123, OUT)

        self.switch.unset_vlan_access_group(123, OUT)

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(123, access_group_out=None)]))

    def test_set_vlan_vrf(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(123)])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive("set_vlan_vrf").once() \
            .with_args(123, 'vrf-name')

        self.switch.set_vlan_vrf(123, 'vrf-name')

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(123, vrf_forwarding='vrf-name')]))

    def test_unset_vlan_vrf(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(123, vrf_forwarding='vrf-name')])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive("unset_vlan_vrf").once() \
            .with_args(123)

        self.switch.unset_vlan_vrf(123)

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(123, vrf_forwarding=None)]))

    def test_set_interface_description(self):
        self.real_switch_mock.should_receive("set_interface_description").once() \
            .with_args('xe-1/0/2', 'interface-description')
        self.switch.set_interface_description('xe-1/0/2', 'interface-description')

    def test_unset_interface_description(self):
        self.real_switch_mock.should_receive("unset_interface_description").once() \
            .with_args('xe-1/0/2')
        self.switch.unset_interface_description('xe-1/0/2')

    def test_set_interface_spanning_tree(self):
        self.real_switch_mock.should_receive("edit_interface_spanning_tree").once() \
            .with_args('xe-1/0/2', edge=None)

        self.switch.edit_interface_spanning_tree('xe-1/0/2')

    def test_add_bond_first(self):
        all_bonds = [Bond(1), Bond(2), Bond(123)]

        self.real_switch_mock.should_receive("add_bond").once().with_args(123)
        self.switch.add_bond(123)

        self.real_switch_mock.should_receive("get_bond").once().and_return(Bond(123))
        assert_that(self.switch.get_bond(123).number, is_(123))

        self.real_switch_mock.should_receive("get_bonds").once().and_return(
            all_bonds)
        assert_that(self.switch.get_bonds(), is_(all_bonds))
        assert_that(self.switch.get_bonds(), is_(all_bonds))

    def test_add_bond_after_get_bonds(self):
        all_bonds = [Bond(1), Bond(2)]
        self.real_switch_mock.should_receive("get_bonds").once().and_return(
            all_bonds)
        assert_that(self.switch.get_bonds(), is_(all_bonds))
        assert_that(self.switch.get_bonds(), is_(all_bonds))

        self.real_switch_mock.should_receive("add_bond").once().with_args(123)
        self.switch.add_bond(123)

        self.real_switch_mock.should_receive("get_bond").once().and_return(Bond(123))
        assert_that(self.switch.get_bond(123).number, is_(123))

        assert_that(
            self.switch.get_bonds(),
            is_(all_bonds+[Bond(123)]))

    def test_remove_bond(self):
        self.real_switch_mock.should_receive("get_bonds").once().and_return(
            [Bond(1), Bond(2)])
        self.switch.get_bonds()

        self.real_switch_mock.should_receive("remove_bond").once().with_args(2)
        self.switch.remove_bond(2)

        assert_that(self.switch.get_bonds(), is_([Bond(1)]))

    def test_remove_bond_alone(self):
        self.real_switch_mock.should_receive("remove_bond").once().with_args(2)
        self.switch.remove_bond(2)

    def test_get_bonds(self):
        all_bonds = [Bond(1), Bond(2)]

        self.real_switch_mock.should_receive("get_bonds").once().and_return(
            all_bonds)
        assert_that(self.switch.get_bonds(), is_(all_bonds))
        assert_that(self.switch.get_bonds(), is_(all_bonds))

    def test_get_bond(self):
        bond = Bond(2)
        all_bonds = [Bond(1), bond]

        self.real_switch_mock.should_receive("get_bond").once().with_args(2) \
            .and_return(bond)
        assert_that(self.switch.get_bond(2), is_(bond))
        assert_that(self.switch.get_bond(2), is_(bond))

        self.real_switch_mock.should_receive("get_bonds").once().and_return(
            all_bonds)
        assert_that(self.switch.get_bonds(), is_(all_bonds))
        assert_that(self.switch.get_bonds(), is_(all_bonds))

    def test_add_interface_to_bond(self):
        self.real_switch_mock.should_receive("get_bonds").once() \
            .and_return([Bond(1)])
        self.real_switch_mock.should_receive("get_interfaces").once() \
            .and_return([Interface('xe-1/0/2')])

        self.switch.get_bonds()
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("add_interface_to_bond").once() \
            .with_args('xe-1/0/2', 1)

        self.switch.add_interface_to_bond('xe-1/0/2', 1)

        self.real_switch_mock.should_receive("get_interfaces").once() \
            .and_return([Interface('xe-1/0/2', bond_master=1, port_mode=BOND_MEMBER)])

        assert_that(
            self.switch.get_bonds(),
            is_([Bond(1, members=['xe-1/0/2'])]))
        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('xe-1/0/2', bond_master=1, port_mode=BOND_MEMBER)]))

    def test_remove_interface_from_bond(self):
        self.real_switch_mock.should_receive("get_bonds").once() \
            .and_return([Bond(1, members=['xe-1/0/2'])])
        self.real_switch_mock.should_receive("get_interfaces").once() \
            .and_return([Interface('xe-1/0/2', bond_master=1)])

        self.switch.get_bonds()
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("remove_interface_from_bond").once() \
            .with_args('xe-1/0/2')

        self.switch.remove_interface_from_bond('xe-1/0/2')

        self.real_switch_mock.should_receive("get_interfaces").once() \
            .and_return([Interface('xe-1/0/2', bond_master=None, port_mode=ACCESS)])

        assert_that(
            self.switch.get_bonds(),
            is_([Bond(1, members=[])]))
        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('xe-1/0/2', bond_master=None, port_mode=ACCESS)]))

    def test_set_bond_link_speed(self):
        self.real_switch_mock.should_receive("get_bonds").once().and_return(
            [Bond(1)])
        self.switch.get_bonds()

        self.real_switch_mock.should_receive("set_bond_link_speed").once() \
            .with_args(1, 'super-fast')

        self.switch.set_bond_link_speed(1, 'super-fast')

        assert_that(
            self.switch.get_bonds(),
            is_([Bond(1, link_speed='super-fast')])
        )

    def test_set_bond_description(self):
        self.real_switch_mock.should_receive("set_bond_description").once() \
            .with_args(312, 'bond-description')
        self.switch.set_bond_description(312, 'bond-description')

    def test_unset_bond_description(self):
        self.real_switch_mock.should_receive("unset_bond_description").once() \
            .with_args(312)
        self.switch.unset_bond_description(312)

    def test_set_bond_trunk_mode(self):
        self.real_switch_mock.should_receive("get_bonds").once().and_return(
            [Bond(1)])
        self.switch.get_bonds()

        self.real_switch_mock.should_receive("set_bond_trunk_mode").once() \
            .with_args(1)

        self.switch.set_bond_trunk_mode(1)

        assert_that(
            self.switch.get_bonds(),
            is_([Bond(1, port_mode=TRUNK)])
        )

    def test_set_bond_access_mode(self):
        self.real_switch_mock.should_receive("get_bonds").once().and_return(
            [Bond(1)])
        self.switch.get_bonds()

        self.real_switch_mock.should_receive("set_bond_access_mode").once() \
            .with_args(1)

        self.switch.set_bond_access_mode(1)

        assert_that(
            self.switch.get_bonds(),
            is_([Bond(1, port_mode=ACCESS)])
        )

    def test_add_bond_trunk_vlan_first(self):
        self.real_switch_mock.should_receive("add_bond_trunk_vlan").once() \
            .with_args(1, 2)

        self.switch.add_bond_trunk_vlan(1, 2)

        self.real_switch_mock.should_receive("get_bond").once().with_args(1) \
            .and_return(Bond(1))
        assert_that(self.switch.get_bond(1), is_(Bond(1)))
        assert_that(self.switch.get_bond(1), is_(Bond(1)))

    def test_add_bond_trunk_vlan_after_get_bonds(self):
        all_bonds = [Bond(1),
                     Bond(2)]

        self.real_switch_mock.should_receive("get_bonds").once().and_return(
            all_bonds)
        assert_that(self.switch.get_bonds(), is_(all_bonds))

        self.real_switch_mock.should_receive("add_bond_trunk_vlan").once() \
            .with_args(1, 2)

        self.switch.add_bond_trunk_vlan(1, 2)

        assert_that(
            self.switch.get_bond(1),
            is_(Bond(1, trunk_vlans=[2]))
        )
        assert_that(
            self.switch.get_bonds(),
            is_([
                Bond(1, trunk_vlans=[2]),
                Bond(2)
            ])
        )

    def test_remove_bond_trunk_vlan(self):
        all_bonds = [Bond(1, trunk_vlans=[2]),
                     Bond(2)]

        self.real_switch_mock.should_receive("get_bonds").once().and_return(
            all_bonds)
        assert_that(self.switch.get_bonds(), is_(all_bonds))

        self.real_switch_mock.should_receive("remove_bond_trunk_vlan").once() \
            .with_args(1, 2)

        self.switch.remove_bond_trunk_vlan(1, 2)

        assert_that(
            self.switch.get_bond(1),
            is_(Bond(1, trunk_vlans=[]))
        )
        assert_that(
            self.switch.get_bonds(),
            is_([
                Bond(1, trunk_vlans=[]),
                Bond(2)
            ])
        )

    def test_remove_bond_trunk_vlan_on_bond_not_in_cache(self):
        self.real_switch_mock.should_receive("remove_bond_trunk_vlan").once() \
            .with_args(1, 2)

        self.switch.remove_bond_trunk_vlan(1, 2)

    def test_set_bond_native_vlan(self):
        self.real_switch_mock.should_receive("get_bonds").once() \
            .and_return([Bond(2)])
        self.switch.get_bonds()

        self.real_switch_mock.should_receive("set_bond_native_vlan").once() \
            .with_args(2, 20)

        self.switch.set_bond_native_vlan(2, 20)

        assert_that(
            self.switch.get_bonds(),
            is_([Bond(2, trunk_native_vlan=20)]))

    def test_unset_bond_native_vlan(self):
        self.real_switch_mock.should_receive("get_bonds").once() \
            .and_return([Bond(2, trunk_native_vlan=20)])
        self.switch.get_bonds()

        self.real_switch_mock.should_receive("unset_bond_native_vlan").once() \
            .with_args(2)

        self.switch.unset_bond_native_vlan(2)

        assert_that(
            self.switch.get_bonds(),
            is_([Bond(2, trunk_native_vlan=None)]))

    def test_edit_bond_spanning_tree(self):
        self.real_switch_mock.should_receive("edit_bond_spanning_tree").once() \
            .with_args(2, edge=None)

        self.switch.edit_bond_spanning_tree(2)

    def test_add_vrrp_group(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(1)])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive("add_vrrp_group").once() \
            .with_args(1, 2, ips=None, priority=23, hello_interval=None,
                       dead_interval=None, track_id=None, track_decrement=None)

        self.switch.add_vrrp_group(1, 2, priority=23)

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(1, vrrp_groups=[VrrpGroup(id=2, priority=23)])]))

    def test_remove_vrrp_group(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(1, vrrp_groups=[VrrpGroup(id=2)])])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive("remove_vrrp_group").once() \
            .with_args(1, 2)

        self.switch.remove_vrrp_group(1, 2)

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(1, vrrp_groups=[])]))

    def test_add_vlan_varp_ip(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(1)])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive("add_vlan_varp_ip").once() \
            .with_args(1, IPNetwork("1.1.1.1/29"))

        self.switch.add_vlan_varp_ip(1, IPNetwork("1.1.1.1/29"))

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(1, varp_ips=[IPNetwork("1.1.1.1/29")])]))

    def test_remove_vlan_varp_ip(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(1, varp_ips=[IPNetwork("1.1.1.1/29")])])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive("remove_vlan_varp_ip").once() \
            .with_args(1, IPNetwork("1.1.1.1/29"))

        self.switch.remove_vlan_varp_ip(1, IPNetwork("1.1.1.1/29"))

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(1, varp_ips=[])]))

    def test_add_dhcp_relay_server(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(2)])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive("add_dhcp_relay_server").once() \
            .with_args(2, IPAddress("1.2.3.4"))

        self.switch.add_dhcp_relay_server(2, IPAddress("1.2.3.4"))

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(2, dhcp_relay_servers=[IPAddress("1.2.3.4")])]))

    def test_set_interface_lldp_state(self):
        self.real_switch_mock.should_receive("set_interface_lldp_state").once() \
            .with_args('xe-1/0/2', True)

        self.switch.set_interface_lldp_state('xe-1/0/2', True)

    def test_set_vlan_arp_routing_state(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(2)])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive('set_vlan_arp_routing_state').once() \
            .with_args(2, OFF)

        self.switch.set_vlan_arp_routing_state(2, OFF)

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(2, arp_routing=False)]))

    def test_set_vlan_icmp_redirects_state(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(2)])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive('set_vlan_icmp_redirects_state').once() \
            .with_args(2, False)

        self.switch.set_vlan_icmp_redirects_state(2, False)

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(2, icmp_redirects=False)]))

    def test_set_vlan_unicast_rpf_mode(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(2)])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive('set_vlan_unicast_rpf_mode').once() \
            .with_args(2, STRICT)

        self.switch.set_vlan_unicast_rpf_mode(2, STRICT)

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(2, unicast_rpf_mode=STRICT)]))

    def test_unset_vlan_unicast_rpf_mode(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(2, unicast_rpf_mode=STRICT)])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive('unset_vlan_unicast_rpf_mode').once() \
            .with_args(2)

        self.switch.unset_vlan_unicast_rpf_mode(2)

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(2, unicast_rpf_mode=None)]))

    def test_get_versions(self):
        self.real_switch_mock.should_receive("get_versions").once().and_return({"v": "1.0"})

        result1 = self.switch.get_versions()
        result2 = self.switch.get_versions()

        assert_that(result1, is_({"v": "1.0"}))
        assert_that(result2, is_({"v": "1.0"}))

    def test_set_interface_mtu(self):
        self.real_switch_mock.should_receive("get_interfaces").once() \
            .and_return([Interface('xe-1/0/2')])
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("set_interface_mtu").once() \
            .with_args('xe-1/0/2', 5000)

        self.switch.set_interface_mtu('xe-1/0/2', 5000)

        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('xe-1/0/2', mtu=5000)]))

    def test_unset_interface_mtu(self):
        self.real_switch_mock.should_receive("get_interfaces").once() \
            .and_return([Interface('xe-1/0/2', mtu=5000)])
        self.switch.get_interfaces()

        self.real_switch_mock.should_receive("unset_interface_mtu").once() \
            .with_args('xe-1/0/2')

        self.switch.unset_interface_mtu('xe-1/0/2')

        assert_that(
            self.switch.get_interfaces(),
            is_([Interface('xe-1/0/2', mtu=None)]))

    def test_set_bond_mtu(self):
        self.real_switch_mock.should_receive("get_bonds").once() \
            .and_return([Bond('xe-1/0/2')])
        self.switch.get_bonds()

        self.real_switch_mock.should_receive("set_bond_mtu").once() \
            .with_args('xe-1/0/2', 5000)

        self.switch.set_bond_mtu('xe-1/0/2', 5000)

        assert_that(
            self.switch.get_bonds(),
            is_([Bond('xe-1/0/2', mtu=5000)]))

    def test_unset_bond_mtu(self):
        self.real_switch_mock.should_receive("get_bonds").once() \
            .and_return([Bond('xe-1/0/2', mtu=5000)])
        self.switch.get_bonds()

        self.real_switch_mock.should_receive("unset_bond_mtu").once() \
            .with_args('xe-1/0/2')

        self.switch.unset_bond_mtu('xe-1/0/2')

        assert_that(
            self.switch.get_bonds(),
            is_([Bond('xe-1/0/2', mtu=None)]))

    def test_set_vlan_load_interval(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(123)])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive("set_vlan_load_interval").once() \
            .with_args(123, 30)
        self.switch.set_vlan_load_interval(123, 30)

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(123, load_interval=30)]))

    def test_unset_vlan_load_interval(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(123, load_interval=30)])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive("unset_vlan_load_interval").once() \
            .with_args(123)
        self.switch.unset_vlan_load_interval(123)

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(123, load_interval=None)]))

    def test_set_vlan_mpls_ip_state_true(self):
        self.real_switch_mock.should_receive("get_vlans").once() \
            .and_return([Vlan(123, mpls_ip=False)])
        self.switch.get_vlans()

        self.real_switch_mock.should_receive("set_vlan_mpls_ip_state").once() \
            .with_args(123, True)
        self.switch.set_vlan_mpls_ip_state(123, True)

        assert_that(
            self.switch.get_vlans(),
            is_([Vlan(123, mpls_ip=True)]))



import unittest

import mock
import pyeapi
from flexmock import flexmock, flexmock_teardown
from hamcrest import assert_that, has_length, equal_to, is_, contains_string
from netaddr import IPNetwork, IPAddress
from pyeapi.eapilib import CommandError

from netman.adapters.switches import arista
from netman.adapters.switches.arista import Arista, parse_vlan_ranges
from netman.core.objects.exceptions import BadVlanNumber, VlanAlreadyExist, BadVlanName, UnknownVlan, \
    UnknownIP, IPNotAvailable, IPAlreadySet, UnknownInterface, UnknownDhcpRelayServer, DhcpRelayServerAlreadyExists, \
    UnknownBond, VarpAlreadyExistsForVlan, VarpDoesNotExistForVlan, BadLoadIntervalNumber, BadMplsIpState
from netman.core.objects.interface import Interface
from netman.core.objects.interface_states import ON, OFF
from netman.core.objects.port_modes import TRUNK
from netman.core.objects.switch_descriptor import SwitchDescriptor
from netman.core.objects.vlan import Vlan
from tests import ignore_deprecation_warnings
from tests.fixtures.arista import vlan_data, result_payload, interface_vlan_data, show_interfaces, interface_address, \
    switchport_data, interface_data


class AristaTest(unittest.TestCase):
    def setUp(self):
        self.switch = Arista(SwitchDescriptor(model='arista', hostname="my.hostname"), transport="Eytch tea tea pee")
        self.switch.node = flexmock()

    def tearDown(self):
        flexmock_teardown()

    def test_get_vlans(self):
        vlans_payload = {'vlans': {'1': vlan_data(name='default'),
                                   '123': vlan_data(name='VLAN0123'),
                                   '456': vlan_data(name='Patate'),
                                   '789': vlan_data(name="new_interface_vlan_without_ip"),
                                   '1234': vlan_data(name="interface_with_removed_ip"),
                                   '1235': vlan_data(name="vlan_with_load_interval"),
                                   '1236': vlan_data(name="vlan_with_no_mpls_ip")}}

        interfaces_payload = show_interfaces(
            interface_vlan_data(name="Vlan456", interfaceAddress=[interface_address(
                primaryIp={'address': '192.168.11.1', 'maskLen': 29},
                secondaryIpsOrderedList=[
                    {'address': '192.168.13.1', 'maskLen': 29},
                    {'address': '192.168.12.1', 'maskLen': 29}
                ]
            )]),
            interface_vlan_data(name="Vlan789", interfaceAddress=[interface_address(
                primaryIp={'address': '0.0.0.0', 'maskLen': 0},
                secondaryIpsOrderedList=[]
            )]),
            interface_vlan_data(name="Vlan1234", interfaceAddress=[]),
        )

        self.switch.node.should_receive("enable") \
            .with_args(["show vlan", "show interfaces"], strict=True) \
            .and_return([result_payload(result=vlans_payload),
                         result_payload(result=interfaces_payload)])

        self.switch.node.should_receive("get_config").with_args(
            params="interfaces Vlan1 Vlan123 Vlan1234 Vlan1235 Vlan1236 Vlan456 Vlan789").once() \
            .and_return(['interface Vlan123',
                         '   ip helper-address 10.10.30.200',
                         '   ip helper-address 10.10.30.201',
                         'interface Vlan456',
                         '   ip virtual-router address 10.10.77.1',
                         '   ip virtual-router address 10.10.77.200/28',
                         'interface Vlan789',
                         '   ip virtual-router address 10.10.77.3',
                         '   ip helper-address 10.10.30.203',
                         '   ip helper-address 10.10.30.204',
                         '   ip helper-address 10.10.30.205',
                         'interface Vlan1234',
                         'interface Vlan1235',
                         '   load-interval 30',
                         'interface Vlan1236',
                         '   no mpls ip'])

        vlan1, vlan123, vlan456, vlan789, vlan1234, vlan1235, vlan1236 = self.switch.get_vlans()

        assert_that(vlan1.number, equal_to(1))
        assert_that(vlan1.name, equal_to('default'))
        assert_that(vlan1.ips, has_length(0))

        assert_that(vlan123.number, equal_to(123))
        assert_that(vlan123.name, equal_to(None))
        assert_that(vlan123.ips, has_length(0))
        dhcp_ip1, dhcp_ip2 = vlan123.dhcp_relay_servers
        assert_that(dhcp_ip1, is_(IPAddress('10.10.30.200')))
        assert_that(dhcp_ip2, is_(IPAddress('10.10.30.201')))
        assert_that(vlan123.varp_ips, has_length(0))

        assert_that(vlan456.number, equal_to(456))
        assert_that(vlan456.name, equal_to('Patate'))
        assert_that(vlan456.name, equal_to('Patate'))
        ip11, ip13, ip12 = vlan456.ips
        assert_that(ip11, is_(IPNetwork("192.168.11.1/29")))
        assert_that(ip13, is_(IPNetwork("192.168.13.1/29")))
        assert_that(ip12, is_(IPNetwork("192.168.12.1/29")))
        assert_that(vlan456.dhcp_relay_servers, has_length(0))
        varp1, varp2 = vlan456.varp_ips
        assert_that(varp1, is_(IPNetwork("10.10.77.1/32")))
        assert_that(varp2, is_(IPNetwork("10.10.77.200/28")))

        assert_that(vlan789.ips, has_length(0))
        dhcp_ip1, dhcp_ip2, dhcp_ip3 = vlan789.dhcp_relay_servers
        assert_that(dhcp_ip1, is_(IPAddress('10.10.30.203')))
        assert_that(dhcp_ip2, is_(IPAddress('10.10.30.204')))
        assert_that(dhcp_ip3, is_(IPAddress('10.10.30.205')))
        assert_that(vlan789.varp_ips, has_length(1))
        varp_ip = vlan789.varp_ips[0]
        assert_that(varp_ip, is_(IPNetwork('10.10.77.3/32')))

        assert_that(vlan1234.ips, has_length(0))
        assert_that(vlan1234.dhcp_relay_servers, has_length(0))
        assert_that(vlan1234.varp_ips, has_length(0))

        assert_that(vlan1235.number, equal_to(1235))
        assert_that(vlan1235.load_interval, equal_to(30))
        assert_that(vlan1235.mpls_ip, equal_to(True))

        assert_that(vlan1236.number, equal_to(1236))
        assert_that(vlan1236.mpls_ip, equal_to(False))

    def test_get_vlan(self):
        vlans_payload = {'vlans': {'456': vlan_data(name='Patate')}}

        interfaces_payload = show_interfaces(
            interface_vlan_data(name="Vlan456", interfaceAddress=[interface_address(
                primaryIp={'address': '192.168.11.1', 'maskLen': 29},
                secondaryIpsOrderedList=[
                    {'address': '192.168.13.1', 'maskLen': 29},
                    {'address': '192.168.12.1', 'maskLen': 29}
                ]
            )])
        )

        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 456", "show interfaces Vlan456"], strict=True) \
            .and_return([result_payload(result=vlans_payload),
                         result_payload(result=interfaces_payload)])

        self.switch.node.should_receive("get_config").with_args(params="interfaces Vlan456").once() \
            .and_return(['interface Vlan456',
                         '   ip helper-address 10.10.30.200',
                         '   ip virtual-router address 10.10.77.1',
                         '   ip helper-address 10.10.30.201',
                         '   ip virtual-router address 10.10.77.2/28',
                         '   load-interval 30',
                         '   no mpls ip'])

        vlan = self.switch.get_vlan(456)

        assert_that(vlan.number, equal_to(456))
        assert_that(vlan.name, equal_to('Patate'))
        assert_that(vlan.load_interval, equal_to(30))
        assert_that(vlan.mpls_ip, equal_to(False))
        ip11, ip13, ip12 = vlan.ips
        assert_that(ip11, is_(IPNetwork("192.168.11.1/29")))
        assert_that(ip13, is_(IPNetwork("192.168.13.1/29")))
        assert_that(ip12, is_(IPNetwork("192.168.12.1/29")))
        varp1, varp2 = vlan.varp_ips
        assert_that(varp1, is_(IPNetwork("10.10.77.1/32")))
        assert_that(varp2, is_(IPNetwork("10.10.77.2/28")))

        helper_addr1, helper_addr2 = vlan.dhcp_relay_servers
        assert_that(helper_addr1, is_(IPAddress('10.10.30.200')))
        assert_that(helper_addr2, is_(IPAddress('10.10.30.201')))

    def test_get_vlan_invalid_ip_does_not_fail(self):
        vlans_payload = {'vlans': {'456': vlan_data(name='Patate')}}

        interfaces_payload = show_interfaces(
            interface_vlan_data(name="Vlan456")
        )

        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 456", "show interfaces Vlan456"], strict=True) \
            .and_return([result_payload(result=vlans_payload),
                         result_payload(result=interfaces_payload)])

        self.switch.node.should_receive("get_config").with_args(params="interfaces Vlan456").once() \
            .and_return(['interface Vlan456',
                         '   ip helper-address say_what',
                         '   ip helper-address 10.10.30.201'])

        vlan = self.switch.get_vlan(456)

        assert_that(vlan.number, equal_to(456))
        assert_that(vlan.dhcp_relay_servers, has_length(1))
        assert_that(vlan.dhcp_relay_servers[0], is_(IPAddress('10.10.30.201')))

    def test_get_vlan_doesnt_exist(self):
        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 111", "show interfaces Vlan111"], strict=True) \
            .and_raise(CommandError(1000, "CLI command 2 of 3 'show vlan 111' failed: could not run command",
                                    command_error="VLAN 111 not found in current VLAN database"))

        with self.assertRaises(UnknownVlan):
            self.switch.get_vlan(111)

    def test_get_vlan_incorrect(self):
        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 5001", "show interfaces Vlan5001"], strict=True) \
            .and_raise(CommandError(1002, "CLI command 2 of 3 'show vlan 5001' failed: could not run command",
                                    command_error="Invalid input (at token 2: '5001')"))

        with self.assertRaises(BadVlanNumber):
            self.switch.get_vlan(5001)

    def test_get_vlan_without_interface(self):
        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 456", "show interfaces Vlan456"], strict=True) \
            .and_raise(CommandError(1002, "CLI command 3 of 3 'show interfaces Vlan456' failed: invalid command",
                                    command_error="Interface does not exist",
                                    output=[
                                        {},
                                        {'vlans': {'456': vlan_data(name='Patate')}},
                                        {'errors': ['Interface does not exist']}
                                    ]))

        self.switch.node.should_receive("get_config").with_args(params="interfaces Vlan456").once() \
            .and_return(['interface Vlan456'])

        vlan = self.switch.get_vlan(456)

        assert_that(vlan.number, equal_to(456))
        assert_that(vlan.name, equal_to('Patate'))
        assert_that(vlan.ips, has_length(0))

    def test_get_vlan_unknown_error(self):
        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 456", "show interfaces Vlan456"], strict=True) \
            .and_raise(CommandError(1002, "CLI command 3 of 3 'show interfaces Vlan456' failed: invalid command",
                                    command_error="This is unexpected"))

        with self.assertRaises(CommandError):
            self.switch.get_vlan(456)

    def test_add_vlan(self):
        self.switch.node.should_receive("enable").with_args(["show vlan 123"], strict=True) \
            .and_raise(CommandError(1000, 'msg'))

        self.switch.node.should_receive("config").with_args(["vlan 123"]).once()

        self.switch.add_vlan(123)

    def test_add_vlan_with_name(self):
        self.switch.node.should_receive("enable").with_args(["show vlan 123"], strict=True) \
            .and_raise(CommandError(1000, 'msg'))

        self.switch.node.should_receive("config").with_args(["vlan 123", "name gertrude"]).once()

        self.switch.add_vlan(123, "gertrude")

    def test_add_vlan_bad_vlan_number(self):
        with self.assertRaises(BadVlanNumber):
            self.switch.add_vlan(12334)

    def test_add_vlan_already_exits(self):
        self.switch.node.should_receive("enable").with_args(["show vlan 123"], strict=True).once() \
            .and_return(result_payload(result=[{'vlans': {'123': vlan_data(name='VLAN0123')}}]))

        with self.assertRaises(VlanAlreadyExist):
            self.switch.add_vlan(123)

    def test_add_vlan_bad_vlan_name(self):
        self.switch.node.should_receive("enable").with_args(["show vlan 123"], strict=True) \
            .and_raise(CommandError(1000, 'msg'))

        self.switch.node.should_receive("config") \
            .with_args(["vlan 123", "name gertrude invalid name"]) \
            .and_raise(CommandError(1000, "CLI command 4 of 4 'gertrude invalid name' failed: invalid command",
                                    command_error="Invalid input (at token 2: 'invalid')"))

        with self.assertRaises(BadVlanName):
            self.switch.add_vlan(123, "gertrude invalid name")

    def test_remove_vlan(self):
        self.switch.node.should_receive("enable").with_args(["show vlan 123"], strict=True).once()
        self.switch.node.should_receive("config").with_args(["no interface Vlan123", "no vlan 123"]).once()

        self.switch.remove_vlan(123)

    def test_remove_vlan_unknown_vlan(self):
        self.switch.node.should_receive("enable").with_args(["show vlan 123"], strict=True).once().and_raise(
            CommandError(1000, 'msg'))

        with self.assertRaises(UnknownVlan):
            self.switch.remove_vlan(123)

    def test_add_ip(self):
        self.switch = flexmock(spec=self.switch)
        self.switch.should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123))
        self.switch.node.should_receive("config").with_args(["interface vlan 123",
                                                             "ip address 1.2.3.4/29"]).once()

        self.switch.add_ip_to_vlan(123, IPNetwork("1.2.3.4/29"))

    def test_add_ip_no_interface(self):
        self.switch = flexmock(spec=self.switch)
        self.switch.should_receive("get_vlan").with_args(123) \
            .and_raise(UnknownInterface())
        self.switch.should_receive("get_vlan").with_args(123).and_return(Vlan(123))
        self.switch.node.should_receive("config").with_args(["interface vlan 123",
                                                             "ip address 1.2.3.4/29"]).once()

        self.switch.add_ip_to_vlan(123, IPNetwork("1.2.3.4/29"))

    def test_add_another_ip(self):
        self.switch = flexmock(spec=self.switch)
        self.switch.should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123, ips=[IPNetwork("1.1.1.1/29")]))
        self.switch.node.should_receive("config").with_args(["interface vlan 123",
                                                             "ip address 1.2.3.4/29 secondary"]).once()

        self.switch.add_ip_to_vlan(123, IPNetwork("1.2.3.4/29"))

    def test_add_unavailable_ip_raises(self):
        self.switch = flexmock(spec=self.switch)
        self.switch.should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123))
        self.switch.node.should_receive("config").with_args(["interface vlan 123", "ip address 1.2.3.4/29"]) \
            .once().and_raise(CommandError(1000, 'Address 1.2.3.4/29 is already assigned to interface Vlan456'))

        with self.assertRaises(IPNotAvailable) as e:
            self.switch.add_ip_to_vlan(123, IPNetwork("1.2.3.4/29"))

            assert_that(str(e), equal_to("IP 1.2.3.4/29 is not available in this vlan: "
                                         "Address 1.2.3.4/29 is already assigned to interface Vlan456"))

    def test_add_an_ip_already_present_in_the_same_port_raises(self):
        flexmock(self.switch).should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123, ips=[IPNetwork("1.2.3.4/29")]))

        with self.assertRaises(IPAlreadySet):
            self.switch.add_ip_to_vlan(123, IPNetwork("1.2.3.4/29"))

    def test_add_an_ip_already_present_in_the_same_port_secondary_raises(self):
        flexmock(self.switch).should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123, ips=[IPNetwork("1.2.3.4/29"), IPNetwork("1.2.3.5/29")]))

        with self.assertRaises(IPAlreadySet):
            self.switch.add_ip_to_vlan(123, IPNetwork("1.2.3.5/29"))

    def test_add_ip_to_unknown_vlan(self):
        flexmock(self.switch).should_receive("get_vlan").with_args(123).and_raise(UnknownVlan(123))

        with self.assertRaises(UnknownVlan):
            self.switch.add_ip_to_vlan(123, IPNetwork("1.2.3.5/29"))

    def test_remove_lonely_ip(self):
        flexmock(self.switch).should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123, ips=[IPNetwork("1.1.1.1/29")]))
        self.switch.node.should_receive("config").with_args(["interface Vlan123",
                                                             "no ip address 1.1.1.1/29"]).once()

        self.switch.remove_ip_from_vlan(123, IPNetwork("1.1.1.1/29"))

    def test_remove_secondary_ip(self):
        flexmock(self.switch).should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123, ips=[IPNetwork("1.1.1.1/29"), IPNetwork("1.1.2.1/29")]))
        self.switch.node.should_receive("config").with_args(["interface Vlan123",
                                                             "no ip address 1.1.2.1/29 secondary"]).once()

        self.switch.remove_ip_from_vlan(123, IPNetwork("1.1.2.1/29"))

    def test_cant_remove_unknown_ip(self):
        flexmock(self.switch).should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123, ips=[IPNetwork("1.1.2.1/29"), IPNetwork("1.1.3.1/29")]))

        with self.assertRaises(UnknownIP) as expect:
            self.switch.remove_ip_from_vlan(123, IPNetwork("1.1.1.1/29"))

        assert_that(str(expect.exception), contains_string("IP 1.1.1.1/29 not found"))

    def test_remove_a_primary_ip_that_have_secondary_ips(self):
        flexmock(self.switch).should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123, ips=[IPNetwork("1.1.1.1/29"), IPNetwork("1.1.2.1/29"), IPNetwork("1.1.3.1/29")]))
        self.switch.node.should_receive("config").with_args(["interface Vlan123",
                                                             "ip address 1.1.2.1/29"]).once()

        self.switch.remove_ip_from_vlan(123, IPNetwork("1.1.1.1/29"))

    def test_cant_remove_known_ip_with_wrong_netmask(self):
        flexmock(self.switch).should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123, ips=[IPNetwork("1.1.1.1/29"), IPNetwork("1.1.2.1/29")]))

        with self.assertRaises(UnknownIP) as expect:
            self.switch.remove_ip_from_vlan(123, IPNetwork("1.1.1.1/30"))

        assert_that(str(expect.exception), contains_string("IP 1.1.1.1/30 not found"))

    def test_remove_ip_from_unknown_vlan(self):
        flexmock(self.switch).should_receive("get_vlan").with_args(123) \
            .and_raise(UnknownVlan(123))

        with self.assertRaises(UnknownVlan):
            self.switch.remove_ip_from_vlan(123, IPNetwork("1.1.1.1/30"))

    def test_transactions_commit_write_memory(self):
        self.switch.node.should_receive("enable").with_args("write memory").once()

        self.switch.commit_transaction()

    def test_transactions_rollback_does_nothing(self):
        self.switch.rollback_transaction()

    def test_get_interface(self):
        interface_payload = result_payload(result={
            'interfaces': {
                'Ethernet1': interface_data(
                    name="Ethernet1",
                    lineProtocolStatus="up",
                    autoNegotiate="off",
                    mtu=1234
                )
            }
        })

        switchport_payload = result_payload(result={
            'switchports': {
                'Ethernet1': {
                    'enabled': True,
                    'switchportInfo': switchport_data(
                        mode="trunk",
                        trunkAllowedVlans="800-802,804",
                        trunkingNativeVlanId=1
                    )
                }
            }
        })

        self.switch.node.should_receive("enable") \
            .with_args(["show interfaces Ethernet1", "show interfaces Ethernet1 switchport"], strict=True) \
            .and_return([interface_payload, switchport_payload])

        interface = self.switch.get_interface("Ethernet1")

        assert_that(interface.name, equal_to("Ethernet1"))
        assert_that(interface.shutdown, equal_to(False))
        assert_that(interface.port_mode, equal_to(TRUNK))
        assert_that(interface.trunk_vlans, equal_to([800, 801, 802, 804]))
        assert_that(interface.auto_negotiation, equal_to(OFF))
        assert_that(interface.mtu, equal_to(1234))

    def test_get_interface_no_interface_raises(self):
        self.switch.node.should_receive("enable") \
            .with_args(["show interfaces InvalidInterface", "show interfaces InvalidInterface switchport"], strict=True) \
            .and_raise(CommandError(1002,
                                    "CLI command 4 of 4 'show interfaces InvalidInterface' failed: invalid command",
                                    command_error="Invalid input (at token 0: 'show')"))

        with self.assertRaises(UnknownInterface):
            self.switch.get_interface("InvalidInterface")

    def test_get_interface_unsupported_interface_raises(self):
        interface_payload = result_payload(result={
            'interfaces': {
                'UndesiredInterface': interface_data(name="UndesiredInterface", )
            }
        })
        switchport_payload = result_payload(result={'switchports': {}})
        self.switch.node.should_receive("enable") \
            .with_args(["show interfaces Unsupported1", "show interfaces Unsupported1 switchport"], strict=True) \
            .and_return([interface_payload, switchport_payload])

        with self.assertRaises(UnknownInterface):
            self.switch.get_interface("Unsupported1")

    def test_get_interfaces(self):
        interface_payload = result_payload(result={
            'interfaces': {
                'Ethernet1': interface_data(
                    name="Ethernet1",
                    lineProtocolStatus="up",
                    autoNegotiate="off",
                    mtu=1234
                ),
                'Ethernet2': interface_data(
                    name="Ethernet2",
                    lineProtocolStatus="down",
                    autoNegotiate="on"
                )}})

        switchport_payload = result_payload(result={
            'switchports': {
                'Ethernet1': {
                    'enabled': True,
                    'switchportInfo': switchport_data(
                        mode="trunk",
                        trunkAllowedVlans="800-802,804",
                        trunkingNativeVlanId=1
                    )
                }
            }
        })

        self.switch.node.should_receive("enable") \
            .with_args(["show interfaces", "show interfaces switchport"], strict=True) \
            .and_return([interface_payload, switchport_payload])

        if1, if2 = sorted(self.switch.get_interfaces(), key=lambda i: i.name)

        assert_that(if1.name, equal_to("Ethernet1"))
        assert_that(if1.shutdown, equal_to(False))
        assert_that(if1.port_mode, equal_to(TRUNK))
        assert_that(if1.trunk_vlans, equal_to([800, 801, 802, 804]))
        assert_that(if1.auto_negotiation, equal_to(OFF))
        assert_that(if1.mtu, equal_to(1234))

        assert_that(if2.name, equal_to("Ethernet2"))
        assert_that(if2.shutdown, equal_to(True))
        assert_that(if2.trunk_vlans, equal_to([]))
        assert_that(if2.auto_negotiation, equal_to(ON))

    def parse_range_test(self):
        result = parse_vlan_ranges(None)
        assert_that(list(result), equal_to(range(1, 4094)))

        result = parse_vlan_ranges("NONE")
        assert_that(list(result), equal_to([]))

        result = parse_vlan_ranges("ALL")
        assert_that(list(result), equal_to(range(1, 4094)))

        result = parse_vlan_ranges("1")
        assert_that(list(result), equal_to([1]))

        result = parse_vlan_ranges("2-5")
        assert_that(list(result), equal_to([2, 3, 4, 5]))

        result = parse_vlan_ranges("1,3-5,7")
        assert_that(list(result), equal_to([1, 3, 4, 5, 7]))

    def test_set_trunk_mode_initial(self):
        self.switch.node.should_receive("config").with_args([
            "interface Ethernet1",
            "switchport mode trunk",
            "switchport trunk allowed vlan none"
        ]).once()

        self.switch.set_trunk_mode("Ethernet1")

    def test_set_trunk_mode_initial_invalid_interface_raises(self):
        self.switch.node.should_receive("config") \
            .with_args(["interface Invalid_Ethernet",
                        "switchport mode trunk",
                        "switchport trunk allowed vlan none"]) \
            .and_raise(CommandError(1002, "CLI command 3 of 6 'interface Invalid_Ethernet' failed: invalid command",
                                    command_error="Invalid input (at token 1: 'Invalid_Ethernet')"))

        with self.assertRaises(UnknownInterface):
            self.switch.set_trunk_mode("Invalid_Ethernet")

    def test_add_trunk_vlan(self):
        self.switch = flexmock(self.switch)
        self.switch.should_receive("get_vlan").with_args(800).and_return(Vlan(800))
        self.switch.node.should_receive("config") \
            .with_args(["interface Ethernet1",
                        "switchport trunk allowed vlan add 800"]).once()

        self.switch.add_trunk_vlan("Ethernet1", vlan=800)

    def test_add_trunk_vlan_invalid_vlan_raises(self):
        self.switch = flexmock(self.switch)
        self.switch.should_receive("get_vlan").with_args(800).and_raise(UnknownVlan(800))

        with self.assertRaises(UnknownVlan):
            self.switch.add_trunk_vlan("Ethernet1", vlan=800)

    def test_add_trunk_vlan_invalid_interface_raises(self):
        self.switch = flexmock(self.switch)
        self.switch.should_receive("get_vlan").with_args(800).and_return(Vlan(800))
        self.switch.node.should_receive("config") \
            .with_args(["interface Invalid_Ethernet",
                        "switchport trunk allowed vlan add 800"]) \
            .and_raise(UnknownInterface("Invalid_Ethernet"))

        with self.assertRaises(UnknownInterface):
            self.switch.add_trunk_vlan("Invalid_Ethernet", vlan=800)

    def test_remove_trunk_vlan(self):
        interface_payload = result_payload(result={'interfaces': {'Ethernet1': interface_data(
            name="Ethernet1",
            lineProtocolStatus="up",
            autoNegotiate="off",
            mtu=1234
        )}})
        switchport_payload = result_payload(result={'switchports': {'Ethernet1': {'enabled': True,
                                                                                  'switchportInfo': switchport_data(
                                                                                      mode="trunk",
                                                                                      trunkAllowedVlans="800-802,804",
                                                                                      trunkingNativeVlanId=1
                                                                                  )}}})
        self.switch.node.should_receive("enable") \
            .with_args(["show interfaces Ethernet1", "show interfaces Ethernet1 switchport"], strict=True) \
            .and_return([interface_payload, switchport_payload])
        self.switch.node.should_receive("config") \
            .with_args(["interface Ethernet1", "switchport trunk allowed vlan remove 800"])

        self.switch.remove_trunk_vlan("Ethernet1", 800)

    def test_remove_trunk_vlan_invalid_vlan_raises(self):
        self.switch = flexmock(self.switch)
        self.switch.should_receive("get_interface") \
            .with_args("Ethernet1") \
            .and_return(Interface(name="Ethernet1", trunk_vlans=[], port_mode="trunk"))

        with self.assertRaises(UnknownVlan):
            self.switch.remove_trunk_vlan("Ethernet1", vlan=800)

    def test_remove_trunk_vlan_invalid_interface_raises(self):
        self.switch = flexmock(self.switch)
        self.switch.should_receive("get_interface") \
            .with_args("Ethernet1") \
            .and_raise(UnknownInterface("Ethernet1"))

        with self.assertRaises(UnknownInterface):
            self.switch.remove_trunk_vlan("Ethernet1", vlan=800)

    def test_remove_trunk_vlan_no_port_mode_still_working(self):
        self.switch = flexmock(self.switch)
        self.switch.should_receive("get_interface") \
            .with_args("Ethernet1") \
            .and_return(Interface(name="Ethernet1", trunk_vlans=[800], port_mode=None))
        self.switch.node.should_receive("config") \
            .with_args(["interface Ethernet1", "switchport trunk allowed vlan remove 800"])

        self.switch.remove_trunk_vlan("Ethernet1", 800)

    def test_add_dhcp_relay_server(self):
        vlans_payload = {'vlans': {'123': vlan_data(name='Patate')}}

        interfaces_payload = show_interfaces(
            interface_vlan_data(name="Vlan123")
        )

        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 123", "show interfaces Vlan123"], strict=True) \
            .and_return([result_payload(result=vlans_payload),
                         result_payload(result=interfaces_payload)])

        self.switch.node.should_receive("get_config").once() \
            .with_args(params="interfaces Vlan123") \
            .and_return(['interface Vlan123',
                         '   ip helper-address 10.10.30.200',
                         '   ip helper-address 10.10.30.201'])

        self.switch.node.should_receive("config").once() \
            .with_args(['interface Vlan123',
                        'ip helper-address 10.10.30.202'])

        self.switch.add_dhcp_relay_server(123, IPAddress('10.10.30.202'))

    def test_add_same_dhcp_relay_server_fails(self):
        vlans_payload = {'vlans': {'123': vlan_data(name='Patate')}}

        interfaces_payload = show_interfaces(
            interface_vlan_data(name="Vlan123")
        )

        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 123", "show interfaces Vlan123"], strict=True) \
            .and_return([result_payload(result=vlans_payload),
                         result_payload(result=interfaces_payload)])

        self.switch.node.should_receive("get_config").once() \
            .with_args(params="interfaces Vlan123") \
            .and_return(['interface Vlan123',
                         '   ip helper-address 10.10.30.200',
                         '   ip helper-address 10.10.30.201'])

        with self.assertRaises(DhcpRelayServerAlreadyExists):
            self.switch.add_dhcp_relay_server(123, IPAddress('10.10.30.201'))

    def test_remove_dhcp_relay_server(self):
        vlans_payload = {'vlans': {'123': vlan_data(name='Patate')}}

        interfaces_payload = show_interfaces(
            interface_vlan_data(name="Vlan123")
        )

        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 123", "show interfaces Vlan123"], strict=True) \
            .and_return([result_payload(result=vlans_payload),
                         result_payload(result=interfaces_payload)])

        self.switch.node.should_receive("get_config").once() \
            .with_args(params="interfaces Vlan123") \
            .and_return(['interface Vlan123',
                         '   ip helper-address 10.10.30.200',
                         '   ip helper-address 10.10.30.202'])

        self.switch.node.should_receive("config").once() \
            .with_args(['interface Vlan123',
                        'no ip helper-address 10.10.30.202'])

        self.switch.remove_dhcp_relay_server(123, IPAddress('10.10.30.202'))

    def test_remove_non_existent_dhcp_relay_server_fails(self):
        vlans_payload = {'vlans': {'123': vlan_data(name='Patate')}}

        interfaces_payload = show_interfaces(
            interface_vlan_data(name="Vlan123")
        )

        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 123", "show interfaces Vlan123"], strict=True) \
            .and_return([result_payload(result=vlans_payload),
                         result_payload(result=interfaces_payload)])

        self.switch.node.should_receive("get_config").once() \
            .with_args(params="interfaces Vlan123") \
            .and_return(['interface Vlan123',
                         '   ip helper-address 10.10.30.200',
                         '   ip helper-address 10.10.30.202'])

        with self.assertRaises(UnknownDhcpRelayServer):
            self.switch.remove_dhcp_relay_server(123, IPAddress('10.10.30.222'))

    def test_set_bond_trunk_mode_initial(self):
        self.switch.node.should_receive("config").with_args([
            "interface Port-Channel1",
            "switchport mode trunk",
            "switchport trunk allowed vlan none"
        ]).once()

        self.switch.set_bond_trunk_mode(1)

    def test_set_bond_trunk_mode_initial_invalid_interface_raises(self):
        self.switch.node.should_receive("config") \
            .with_args(["interface Port-Channel9999",
                        "switchport mode trunk",
                        "switchport trunk allowed vlan none"]) \
            .and_raise(CommandError(1002, "CLI command 3 of 6 'interface Port-Channel9999' failed: invalid command",
                                    command_error="Invalid input (at token 1: 'Port-Channel9999')"))

        with self.assertRaises(UnknownBond):
            self.switch.set_bond_trunk_mode(9999)

    def test_add_bond_trunk_vlan(self):
        self.switch = flexmock(self.switch)
        self.switch.should_receive("get_vlan").with_args(800).and_return(Vlan(800))
        self.switch.node.should_receive("config") \
            .with_args(["interface Port-Channel1",
                        "switchport trunk allowed vlan add 800"]).once()

        self.switch.add_bond_trunk_vlan(1, vlan=800)

    def test_add_bond_trunk_vlan_invalid_vlan_raises(self):
        self.switch = flexmock(self.switch)
        self.switch.should_receive("get_vlan").with_args(800).and_raise(UnknownVlan(800))

        with self.assertRaises(UnknownVlan):
            self.switch.add_bond_trunk_vlan(1, vlan=800)

    def test_add_bond_trunk_vlan_invalid_interface_raises(self):
        self.switch = flexmock(self.switch)
        self.switch.should_receive("get_vlan").with_args(800).and_return(Vlan(800))
        self.switch.node.should_receive("config") \
            .with_args(["interface Port-Channel9999",
                        "switchport trunk allowed vlan add 800"]) \
            .and_raise(CommandError(1002, "CLI command 3 of 6 'interface Port-Channel9999' failed: invalid command",
                                    command_error="Invalid input (at token 1: 'Port-Channel9999')"))

        with self.assertRaises(UnknownBond):
            self.switch.add_bond_trunk_vlan(9999, vlan=800)

    def test_remove_bond_trunk_vlan(self):
        self.switch = flexmock(self.switch)
        self.switch.should_receive("get_interface") \
            .with_args("Port-Channel1") \
            .and_return(Interface(name="Port-Channel1", trunk_vlans=[800], port_mode="trunk"))
        self.switch.node.should_receive("config") \
            .with_args(["interface Port-Channel1", "switchport trunk allowed vlan remove 800"])

        self.switch.remove_bond_trunk_vlan(1, 800)

    def test_remove_bond_trunk_vlan_invalid_vlan_raises(self):
        self.switch = flexmock(self.switch)
        self.switch.should_receive("get_interface") \
            .with_args("Port-Channel1") \
            .and_return(Interface(name="Port-Channel1", trunk_vlans=[], port_mode="trunk"))

        with self.assertRaises(UnknownVlan):
            self.switch.remove_bond_trunk_vlan(1, vlan=800)

    def test_remove_bond_trunk_vlan_invalid_interface_raises(self):
        self.switch = flexmock(self.switch)
        self.switch.should_receive("get_interface") \
            .with_args("Port-Channel9999") \
            .and_raise(UnknownInterface("Port-Channel9999"))

        with self.assertRaises(UnknownBond):
            self.switch.remove_bond_trunk_vlan(9999, vlan=800)

    def test_add_varp_ip(self):
        vlans_payload = {'vlans': {'123': vlan_data(name='Patate')}}

        interfaces_payload = show_interfaces(
            interface_vlan_data(name="Vlan123")
        )

        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 123", "show interfaces Vlan123"], strict=True) \
            .and_return([result_payload(result=vlans_payload),
                         result_payload(result=interfaces_payload)])

        self.switch.node.should_receive("get_config").once() \
            .with_args(params="interfaces Vlan123") \
            .and_return(['interface Vlan123'])

        self.switch.node.should_receive("config").once() \
            .with_args(['interface Vlan123',
                        'ip virtual-router address 10.10.20.12/28'])

        self.switch.add_vlan_varp_ip(123, IPNetwork('10.10.20.12/28'))

    def test_add_varp_ip_unknown_vlan_raises(self):
        self.switch.node.should_receive("enable").once() \
            .with_args(["show vlan 111", "show interfaces Vlan111"], strict=True) \
            .and_raise(CommandError(1000, "CLI command 2 of 3 'show vlan 111' failed: could not run command",
                                    command_error="VLAN 111 not found in current VLAN database"))

        with self.assertRaises(UnknownVlan):
            self.switch.add_vlan_varp_ip(111, IPNetwork('10.10.20.12/28'))

    def test_add_existing_varp_ip_to_same_vlan_raises(self):
        vlans_payload = {'vlans': {'123': vlan_data(name='Patate')}}

        interfaces_payload = show_interfaces(
            interface_vlan_data(name="Vlan123")
        )

        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 123", "show interfaces Vlan123"], strict=True) \
            .and_return([result_payload(result=vlans_payload),
                         result_payload(result=interfaces_payload)])

        self.switch.node.should_receive("get_config").once() \
            .with_args(params="interfaces Vlan123") \
            .and_return(['interface Vlan123',
                         '   ip virtual-router address 10.10.20.12/28'])

        with self.assertRaises(VarpAlreadyExistsForVlan):
            self.switch.add_vlan_varp_ip(123, IPNetwork('10.10.20.12/28'))

    def test_add_varp_conflicting_ip_raises(self):
        vlans_payload = {'vlans': {'123': vlan_data(name='Patate')}}

        interfaces_payload = show_interfaces(
            interface_vlan_data(name="Vlan123")
        )

        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 123", "show interfaces Vlan123"], strict=True) \
            .and_return([result_payload(result=vlans_payload),
                         result_payload(result=interfaces_payload)])

        self.switch.node.should_receive("get_config").once() \
            .with_args(params="interfaces Vlan123") \
            .and_return(['interface Vlan123'])

        self.switch.node.should_receive("config").once() \
            .with_args(['interface Vlan123',
                        'ip virtual-router address 10.10.20.12/28']) \
            .and_raise(CommandError(1000,
                                    "Error [1000]: CLI command 4 of 4 "
                                    "'ip virtual-router address 10.10.20.1/28' failed: could not run command "
                                    "[Address 10.10.20.1 is already assigned to interface Vlan20]"))

        with self.assertRaises(IPNotAvailable):
            self.switch.add_vlan_varp_ip(123, IPNetwork('10.10.20.12/28'))

    def test_add_varp_other_eapi_error_raises(self):
        vlans_payload = {'vlans': {'123': vlan_data(name='Patate')}}

        interfaces_payload = show_interfaces(
            interface_vlan_data(name="Vlan123")
        )

        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 123", "show interfaces Vlan123"], strict=True) \
            .and_return([result_payload(result=vlans_payload),
                         result_payload(result=interfaces_payload)])

        self.switch.node.should_receive("get_config").once() \
            .with_args(params="interfaces Vlan123") \
            .and_return(['interface Vlan123'])

        self.switch.node.should_receive("config").once() \
            .with_args(['interface Vlan123',
                        'ip virtual-router address 10.10.20.12/28']) \
            .and_raise(CommandError(1000, "Communication Error"))

        with self.assertRaises(CommandError):
            self.switch.add_vlan_varp_ip(123, IPNetwork('10.10.20.12/28'))

    def test_remove_varp_ip(self):
        vlans_payload = {'vlans': {'123': vlan_data(name='Patate')}}

        interfaces_payload = show_interfaces(
            interface_vlan_data(name="Vlan123")
        )

        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 123", "show interfaces Vlan123"], strict=True) \
            .and_return([result_payload(result=vlans_payload),
                         result_payload(result=interfaces_payload)])

        self.switch.node.should_receive("get_config").once() \
            .with_args(params="interfaces Vlan123") \
            .and_return(['interface Vlan123',
                         '   ip virtual-router address 10.10.20.12/28'])

        self.switch.node.should_receive("config").once() \
            .with_args(['interface Vlan123',
                        'no ip virtual-router address 10.10.20.12/28'])

        self.switch.remove_vlan_varp_ip(123, IPNetwork('10.10.20.12/28'))

    def test_remove_varp_unknown_ip_raises(self):
        vlans_payload = {'vlans': {'123': vlan_data(name='Patate')}}

        interfaces_payload = show_interfaces(
            interface_vlan_data(name="Vlan123")
        )

        self.switch.node.should_receive("enable") \
            .with_args(["show vlan 123", "show interfaces Vlan123"], strict=True) \
            .and_return([result_payload(result=vlans_payload),
                         result_payload(result=interfaces_payload)])

        self.switch.node.should_receive("get_config").once() \
            .with_args(params="interfaces Vlan123") \
            .and_return(['interface Vlan123'])

        with self.assertRaises(VarpDoesNotExistForVlan):
            self.switch.remove_vlan_varp_ip(123, IPNetwork('10.10.20.12/28'))

    def test_remove_varp_ip_unknown_vlan_raises(self):
        self.switch.node.should_receive("enable").once() \
            .with_args(["show vlan 111", "show interfaces Vlan111"], strict=True) \
            .and_raise(CommandError(1000, "CLI command 2 of 3 'show vlan 111' failed: could not run command",
                                    command_error="VLAN 111 not found in current VLAN database"))

        with self.assertRaises(UnknownVlan):
            self.switch.remove_vlan_varp_ip(111, IPNetwork('10.10.20.12/28'))

    def test_set_vlan_load_interval(self):
        self.switch = flexmock(spec=self.switch)
        self.switch.should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123))
        self.switch.node.should_receive("config").with_args(["interface Vlan123",
                                                             "load-interval 30"]).once()

        self.switch.set_vlan_load_interval(123, 30)

    def test_set_vlan_load_interval_unknown_vlan_raises(self):
        self.switch = flexmock(self.switch)
        self.switch.should_receive("get_vlan").with_args(123).and_raise(UnknownVlan(123))

        with self.assertRaises(UnknownVlan):
            self.switch.set_vlan_load_interval(123, 30)

    def test_set_vlan_load_interval_with_bad_number_raises(self):
        self.switch = flexmock(spec=self.switch)
        self.switch.should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123))
        self.switch.node.should_receive("config").with_args(["interface Vlan123", "load-interval 800"]) \
            .and_raise(CommandError(1002, "CLI command 4 of 4 'load-interval 800' failed: invalid command",
                                    command_error="Invalid input"))

        with self.assertRaises(BadLoadIntervalNumber):
            self.switch.set_vlan_load_interval(123, 800)

    def test_unset_vlan_load_interval(self):
        self.switch = flexmock(spec=self.switch)
        self.switch.should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123))
        self.switch.node.should_receive("config").with_args(["interface Vlan123",
                                                             "no load-interval"]).once()

        self.switch.unset_vlan_load_interval(123)

    def test_unset_vlan_load_interval_unknown_vlan_raises(self):
        self.switch = flexmock(self.switch)
        self.switch.should_receive("get_vlan").with_args(123).and_raise(UnknownVlan(123))

        with self.assertRaises(UnknownVlan):
            self.switch.unset_vlan_load_interval(123)

    def test_set_vlan_mpls_ip_state_false(self):
        self.switch = flexmock(spec=self.switch)
        self.switch.should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123))
        self.switch.node.should_receive("config").with_args(["interface Vlan123",
                                                             "no mpls ip"]).once()

        self.switch.set_vlan_mpls_ip_state(123, False)

    def test_set_vlan_mpls_ip_state_true(self):
        self.switch = flexmock(spec=self.switch)
        self.switch.should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123))
        self.switch.node.should_receive("config").with_args(["interface Vlan123",
                                                             "mpls ip"]).once()

        self.switch.set_vlan_mpls_ip_state(123, True)

    def test_set_vlan_mpls_ip_with_invalid_state_raises(self):
        self.switch = flexmock(spec=self.switch)
        self.switch.should_receive("get_vlan").with_args(123) \
            .and_return(Vlan(123))

        with self.assertRaises(BadMplsIpState):
            self.switch.set_vlan_mpls_ip_state(123, 30)

    def test_set_vlan_mpls_ip_unknown_vlan_raises(self):
        self.switch = flexmock(self.switch)
        self.switch.should_receive("get_vlan").with_args(123).and_raise(UnknownVlan(123))

        with self.assertRaises(UnknownVlan):
            self.switch.set_vlan_mpls_ip_state(123, False)


class AristaFactoryTest(unittest.TestCase):
    def tearDown(self):
        flexmock_teardown()

    def test_arista_instance_with_proper_transport(self):
        pyeapi_client_node = mock.sentinel

        flexmock(pyeapi).should_receive('connect').once() \
            .with_args(host="1.2.3.4",
                       username="you sir name",
                       password="paw sword",
                       port=8888,
                       transport="trololo",
                       return_node=True,
                       timeout=300) \
            .and_return(pyeapi_client_node)

        switch = Arista(
            SwitchDescriptor(model='arista',
                             hostname="1.2.3.4",
                             username="you sir name",
                             password="paw sword",
                             port=8888),
            transport="trololo"
        )

        switch._connect()

        assert_that(switch.node, is_(pyeapi_client_node))

    def test_arista_uses_command_timeout(self):
        arista.default_command_timeout = 500

        pyeapi_client_node = mock.sentinel

        flexmock(pyeapi).should_receive('connect').once() \
            .with_args(host="1.2.3.4",
                       username=None,
                       password=None,
                       port=None,
                       transport="trololo",
                       return_node=True,
                       timeout=500) \
            .and_return(pyeapi_client_node)

        switch = Arista(
            SwitchDescriptor(model='arista',
                             hostname="1.2.3.4"),
            transport="trololo"
        )

        switch._connect()

    @ignore_deprecation_warnings
    def test_factory_transport_auto_detection_http(self):
        switch_descriptor = SwitchDescriptor(model="arista", hostname='http://hostname')

        instance = mock.sentinel
        flexmock(arista).should_receive("Arista").once() \
            .with_args(switch_descriptor, transport="http") \
            .and_return(mock.sentinel)

        assert_that(arista.eapi(switch_descriptor), is_(instance))

        assert_that(switch_descriptor.hostname, is_("hostname"))

    @ignore_deprecation_warnings
    def test_factory_transport_auto_detection_https(self):
        switch_descriptor = SwitchDescriptor(model="arista", hostname='https://hostname')

        instance = mock.sentinel
        flexmock(arista).should_receive("Arista").once() \
            .with_args(switch_descriptor, transport="https") \
            .and_return(mock.sentinel)

        assert_that(arista.eapi(switch_descriptor), is_(instance))

        assert_that(switch_descriptor.hostname, is_("hostname"))

    @ignore_deprecation_warnings
    def test_factory_transport_auto_detection_assumes_https_if_not_specified(self):
        switch_descriptor = SwitchDescriptor(model="arista", hostname='hostname')

        instance = mock.sentinel
        flexmock(arista).should_receive("Arista").once() \
            .with_args(switch_descriptor, transport="https") \
            .and_return(mock.sentinel)

        assert_that(arista.eapi(switch_descriptor), is_(instance))

        assert_that(switch_descriptor.hostname, is_("hostname"))
